
/*******************************************************************************
*	DISCLAIMER
*
*	THIS SOFTWARE IS PROVIDED "AS IS" AND BFG ENGINEERING MAKES NO WARRANTIES
*	REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING
*	BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*	PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
*
*	WE (BFG ENGINEERING) DO NOT WARRANT THAT THIS SOFTWARE IS FREE FROM CLAIMS
*	BY A THIRD PARTY OF COPYRIGHT, PATENT, TRADEMARK, TRADE SECRET OR ANY OTHER
*	INTELLECTUAL PROPERTY INFRINGIMENT.
*
*	TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, BFG ENGINEERING
*	SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
*	CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF BFG
*	ENGINEERING HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*	PARTICULARLY, UNDER NO CIRCUMSTANCES ARE WE LIABLE FOR ANY OF THE FOLLOWING:
*	- THIRD PARTY CLAIMS AGAINST YOU FOR LOSSES OR DAMAGES;
*	- LOSS OF, OR DAMAGE TO YOUR RECORDS OR DATA; OR
*	- ECONOMIC CONSEQUENTIAL DAMAGES (INCLUDING LOST PROFITS OR SAVINGS) OR
*	  INCIDENTAL DAMAGES, EVEN IF WE ARE INFORMED OF THEIR POSSIBILITY.
*
*	BFG ENGINEERING DOES NOT WARRANT UNINTERRUPTED OR ERROR_FREE OPERATION OF
*	THIS SOFTWARE.
*	BFG ENGINEERING HAS NO OBLIGATION TO PROVIDE SERVICE, DEFECT CORRECTION OR
*	ANY MANTEINANCE OF THIS SOFTWARE; OR TO SUPPLY ANY UPDATES OR ENHANCEMENTS
*	EVEN IF SUCH ARE, OR LATER BECOME, AVAILABLE.
*
*	IF YOU DOWNLOAD OR USE THIS SOFTWARE YOU AGREE TO THESE TERMS, AND DO IT AT
*	YOUR OWN RISK.
*******************************************************************************/

/*******************************************************************************
*	File:		motorcontrol.c
*	Project:	YRotateItRX111_3s_V1
*	Version:	1.0
*	Date:		01-07-2014 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	motor control algorithm
*******************************************************************************/

#include "globdef.h"
#include "ges_eqp.h"
#include "userif.h"
#include "motorcontrol.h"
#include "mcrplib.h"
#ifdef CPI_AUT
#include "PI_tun.h"
#endif	// CPI_AUT
#ifdef	MOT_IDENT
#include "mot_ident.h"
#endif	// MOT_IDENT

// flags
#define	COM_REQ				(0 != flgx.bit.b0)
#define	FREE01				(0 != flgx.bit.b1)
#define	STOPPED				(0 != flgx.bit.b2)
#define	STARTED				(0 != flgx.bit.b3)
#define	FLGX4				(0 != flgx.bit.b4)
#define	DECEL				(0 != flgx.bit.b5)
#define	RAMPLIM				(0 != flgx.bit.b6)
#define	FLGX7				(0 != flgx.bit.b7)
#define	NOT_COM_REQ			(0 == flgx.bit.b0)
#define	NOT_FREE01			(0 == flgx.bit.b1)
#define	NOT_STOPPED			(0 == flgx.bit.b2)
#define	NOT_STARTED			(0 == flgx.bit.b3)
#define	NOT_FLGX4			(0 == flgx.bit.b4)
#define	NOT_DECEL			(0 == flgx.bit.b5)
#define	NOT_RAMPLIM			(0 == flgx.bit.b6)
#define	NOT_FLGX7			(0 == flgx.bit.b7)
#define	SET_COM_REQ			flgx.bit.b0 = 1;
#define	SET_FREE01			flgx.bit.b1 = 1;
#define	SET_STOPPED			flgx.bit.b2 = 1;
#define	SET_STARTED			flgx.bit.b3 = 1;
#define	SET_FLGX4			flgx.bit.b4 = 1;
#define	SET_DECEL			flgx.bit.b5 = 1;
#define	SET_RAMPLIM			flgx.bit.b6 = 1;
#define	SET_FLGX7			flgx.bit.b7 = 1;
#define	RES_COM_REQ			flgx.bit.b0 = 0;
#define	RES_FREE01			flgx.bit.b1 = 0;
#define	RES_STOPPED			flgx.bit.b2 = 0;
#define	RES_STARTED			flgx.bit.b3 = 0;
#define	RES_FLGX4			flgx.bit.b4 = 0;
#define	RES_DECEL			flgx.bit.b5 = 0;
#define	RES_RAMPLIM			flgx.bit.b6 = 0;
#define	RES_FLGX7			flgx.bit.b7 = 0;

// internal filters time constants
#define SH_OFF_LPKT			14		// current offset filters (2048ms @8kHz)
#define G_OFF_LPKT			(1 << SH_OFF_LPKT)
#define SH_VB_DEC_LPTK		4		// bus voltage filter when decreasing (2ms @8kHz)
#define G_VB_DEC_LPTK		(1 << SH_VB_DEC_LPTK)
#define SH_VB_INC_LPTK		11		// bus voltage filter when increasing (256ms @8kHz)
#define G_VB_INC_LPTK		(1 << SH_VB_INC_LPTK)
#define SH_VB_UP			(SH_VB_INC_LPTK - SH_VB_DEC_LPTK)
#define G_VB_UP				(1 << SH_VB_UP)
#define SH_OMRIF_LPTK		7		// speed reference filter (16ms @8kHz)
#define G_OMRIF_LPTK		(1 << SH_OMRIF_LPTK)
#define SH_VISU_LPTK		12		// visualization filters (512ms @8kHz)
#define G_VISU_LPTK			(1 << SH_VISU_LPTK)

// state values for three shunts current reading
#define	CR_NO				0
#define	CR_UV				1
#define	CR_VW				2
#define	CR_WU				3

// speed ramp state machine
#define STOP_0				0
#define STOP_W				1
#define START_P				2
#define START_N				3
#define GO_P				4
#define GO_N				5

// reference current amplification
#define SH_IDRL				8
#define G_IDRL				(1 << SH_IDRL)
#define GFW_IDRL			((uint16_t)(0.8 * (1 << SH_IDRL)))

// integral gains adaptation
#define AMP_KISF			8000

/******************************************************************************
	Private global variables
******************************************************************************/

byte_t
	flgx,					// flags
	flgy;					// flags

uint16_t
	cnt_int,				// counter for main loop synchronization
	cr_status,				// status memory for current reading
	r_acc_ep,				// acceleration ramp (copy) [rpm/sec]
	r_dec_ep,				// deceleration ramp (copy) [rpm/sec]
	i_start_ep,				// startup current (copy) [A/AMP_DIV]
	i_max_ep,				// maximum current (copy) [A/AMP_DIV]
	r_sta_ep,				// stator resistance (copy) [Ohm/OHM_DIV]
	l_syn_ep,				// synchronous inductance (copy) [Henry/HEN_DIV]
	pm_flx_ep,				// permanent magnets flux (copy)
	st_tim_ep,				// startup acceleration time (eeprom copy) [sec/SEC_DIV]
	pha_off_ep,				// phase offset (copy)
	flx_ftau_ep,			// approx. flux estimation filter time constant tau [sec/SEC_DIV]
	flx_ftau,				// approx. flux estimation filter time constant tau [int]
	aux_ad,					// aux A/D conversion result
	tmp_ad,					// temperature A/D conversion result
	kr_low,					// resistance conversion constant
	sh_kr_low,				// resistance conversion constant amplification shifts
	kl_low,					// inductance conversion constant
	sh_kl_low,				// inductance conversion constant amplification shifts
	kf_low,					// flux conversion constant
	sh_kf_low,				// flux conversion constant amplification shifts
	ramp_status;			// speed ramp state machine status memory

int16_t
	startup_cnt,			// startup counter
	trip_cnt,				// counter for trip alarm
	duty_u,					// imposed u duty cycle
	duty_v,					// imposed v duty cycle
	duty_w,					// imposed w duty cycle
	ium_off,				// IU analog channel offset [bit]
	ivm_off,				// IV analog channel offset [bit]
	iwm_off,				// IW analog channel offset [bit]
	r_sta,					// statoric resistance [internal resistance unit]
	l_syn,					// synchronous inductance [internal inductance unit]
	pm_flx,					// expected permanent magnets flux [int]
	fb_gain,				// flux feedback gain
	c_poli,					// polar couples
	//krpmocp,				// conversion constant: omega[int2]=speed[rpm]*krpmocp/NORMVAL
	korpmdcp,				// conversion constant: speed[rpm]=omega[int2]*korpmdcp/NORMVAL
	i_start,				// startup current [internal current unit]
	delta_om,				// speed variation at every step during startup
	is_rup,					// reference current variation during startup (amplified)
	is_rdw,					// reference current variation during startup (amplified)
	pha_off,				// phase offset [int]
	pha_ofa,				// phase offset (absolute value) [int]
	vbus,					// bus voltage [internal voltage unit]
	vbusf,					// filtered (for visualization) bus voltage [int]
	xvbf,					// filtered bus voltage (control) [internal voltage unit]
	vfmax,					// maximum phase voltage [internal voltage unit]
	i_max,					// maximum current [internal current unit]
	iqmax,					// maximum q current [int]
	vdc,					// requested d voltage [internal voltage unit]
	vqc,					// requested q voltage [internal voltage unit]
	vdcf,					// filtered (for visualization) d voltage [int]
	vqcf,					// filtered (for visualization) q voltage [int]
	vac,					// requested alpha voltage [internal voltage unit]
	vbc,					// requested beta voltage [internal voltage unit]
	vuc,					// requested u voltage [internal voltage unit]
	vvc,					// requested v voltage [internal voltage unit]
	vwc,					// requested w voltage [internal voltage unit]
	ium,					// measured u current [internal current unit]
	ivm,					// measured v current [internal current unit]
	iwm,					// measured w current [internal current unit]
	iam,					// measured alpha current [internal current unit]
	ibm,					// measured beta current [internal current unit]
	idm,					// measured d current [internal current unit]
	iqm,					// measured q current [internal current unit]
	idmf,					// filtered (for visualization) d current [int]
	iqmf,					// filtered (for visualization) q current [int]
	idr,					// reference d current [internal current unit]
	iqr,					// reference q current [internal current unit]
	kp_cur,					// current PI proportional gain
	ki_cur,					// current PI integral gain
	kp_vel,					// speed PI proportional gain
	ki_vel,					// speed PI integral gain
	freq,					// imposed frequency [Hz/10]
	mec_rpm,				// mechanical speed [rpm]
	rif_mem,				// reference speed memory [rpm]
	rpmrif_x,				// reference speed (ramp input) [rpm]
	rpmrif_y,				// reference speed (ramp output) [rpm]
	r_acc,					// acceleration ramp [rpm/main_loop_duration]
	r_dec,					// deceleration ramp [rpm/main_loop_duration]
	rpm_max,				// maximum speed [rpm]
	rpm_min,				// minimum speed [rpm]
	min_speed2,				// minimum speed [in2]
	max_speed2,				// maximum speed [in2]
	Speed_est2,				// estimated speed [in2]
	om_rif2,				// reference angular velocity [in2]
	om_rff2,				// filtered reference angular velocity [in2]
	om_eme2,				// electro-mechanical angular speed [in2]
	om_emf2,				// filtered (visualization) electro mechanical angular velocity [in2]
#ifdef FLUXWEAK
	amp_v,					// requested voltage vector amplitude [int]
	arg_v,					// requested voltage vector phase [int]
	i_del,					// reference current variation during ramp (amplified)
	vmarg,					// voltage margin for flux weakening calc [int]
#endif	// FLUXWEAK
	f_amp_i,				// filtered current vector amplitude
	f_arg_i,				// filtered current vector phase
	f_amp_v,				// filtered voltage vector amplitude
	f_arg_v,				// filtered voltage vector phase
	startup_phase,			// startup phase
	SystemPhase,			// rotating reference system angular position [int]
	Phase_est,				// estimated phase
	FluxAmp,				// flux vector amplitude
	FluxPhase;				// flux vector phase

int32_t
	stup_tim,				// startup acceleration time  [int]
	startup_phasel,			// amplified startup phase [int/NORMVAL]
	ium_ofm,				// IU analog channel offset filter memory
	ivm_ofm,				// IV analog channel offset filter memory
	iwm_ofm,				// IW analog channel offset filter memory
	errint,					// speed PI integral memory
	idint,					// d current PI integral memory
	iqint,					// q current PI integral memory
	xvbm,					// vbus control filter memory
	id_maxl,				// amplified maximum d reference current
	idrl,					// amplified d reference current
	idmm,					// d current visualization filter memory
	iqmm,					// q current visualization filter memory
	vdcm,					// d voltage visualization filter memory
	vqcm,					// q voltage visualization filter memory
	vbusm,					// bus voltage visualization filter memory
	om_eme,					// electro-mechanical angular speed [in1]
	om_rfm2,				// reference angular velocity filter memory
	om_emm2,				// electro mechanical angular velocity visualization filter memory
	om_chg,					// speed at which the speed loop is closed [in1]
	min_speed_trip,			// minimum speed for trip detection [in1]
	max_speed_trip,			// maximum speed for trip detection [in1]
	Speed_est,				// estimated speed [in1]
	om_stup,				// startup speed [in1]
	i_startl;				// amplified startup current

// variables involved in sampling frequency adaptation
float32_t
	pwm_fre_Hz,				// true pwm frequency [Hz]
	//pwm_per_sec,			// true pwm period [sec]
	sam_fre_Hz,				// true sampling frequency [Hz]
	//sam_per_sec,			// true sampling period [sec]
	mloop_fre_Hz,			// true main loop frequency [Hz]
	//mloop_per_sec,		// true main loop period [sec]
	ko1;					// constant used in speed conversion

uint32_t
	halfper_NRM,			// useful in modulation calc
	halfper_amp15;			// useful in modulation calc

uint16_t
	fre_ratio,
	int_skip,
	halfper,
	pwm_per_int,
	sam_per_int,
	num_int,
	sfkHz1024,
	halfperdead,
	centre_pos,
	delmaxp,
	delmaxn,
	kvout,
	ko1_2,					// constant used in speed conversion
	sh_kl_low,
	kl_low,
	sh_kf_low,
	kf_low;

#define pwm_halfper_int		halfper
#define kt					sam_fre_Hz
#define DELMAX( X )			(((int16_t)( X )) >= 0? delmaxp: delmaxn)
// end of variables involved in sampling frequency adjustment

#ifdef CPI_TUN
// current PI tuning
#define I_PIT_RATIO1	((int16_t)(0.8 * NORMVAL))	// 80% of i_max
int16_t
	ratio,
	pit_s,
	pit_c;
int32_t
	cur_ampf;
#endif	// CPI_TUN

#ifdef CPI_AUT
// automatic current PI tuning
#define I_PIT_RATIO2	((int16_t)(0.8 * NORMVAL))	// 80% of i_max
#define R_UP			0
#define R_MS			1
#define R_DW			2
#define R_ST			3
#define R_EN			4
int16_t
	v_req,
	pitun_status;
#endif	// CPI_AUT

// motor parameters identification
#ifdef MOT_IDENT
#define IDENT_SPEED_HZ	20
#define OM_ID2F			(KRPMO * IDENT_SPEED_HZ * 60.0 / NORMVALF)
#define OM_ID2			((uint16_t)OM_ID2F)
#define OM_ID			((uint32_t)(ko1_2 * OM_ID2F))
#define OM_IDR			((uint16_t)(ko1_2 * OM_ID2F / MC_Cntms(SPE_RISE_TIME_MS)))
#define X_TIM1			MC_Cntms( ID_TIM1_MS )	// current rising + alignment + stabilization
#define X_TIM2			MC_Cntms( ID_TIM2_MS )	// first measurement step
#define X_TIM3			MC_Cntms( ID_TIM3_MS )	// current falling + stabilization
#define X_TIM4			MC_Cntms( ID_TIM4_MS )	// second measurement step
#define X_TIM5			MC_Cntms( ID_TIM5_MS )	// speed rising + stabilization
#define X_TIM6			MC_Cntms( ID_TIM6_MS )	// third measurement step
#define X_TIM7			MC_Cntms( ID_TIM7_MS )	// current falling + stabilization
#define X_TIM8			MC_Cntms( ID_TIM8_MS )	// fourth measurement step
#define INI_IDENT		{x_cnt = 0; ial_l = 0; MC_Ident_IniVar(KV, KI, KR, KL, KF, KO2);}
#define IDENT_ENDED		(x_cnt > X_TIM8)
int32_t
	x_cnt = 0;			// identification synchronization counter
int16_t
	iden_rs = 0,
	iden_ls = 0,
	iden_fl = 0,
	ial_r = 0;			// identification current ramp
int32_t
	iden_phasel = 0,	// amplified phase
	om_iden = 0,		// identification speed
	ial_l = 0,			// identification current
	ial_l1 = 0,			// identification current 1
	ial_l2 = 0,			// identification current 2
	ial_l3 = 0;			// identification current 3
#endif	// MOT_IDENT

// oscilloscope window in GUI
#ifdef OSC_WIN
int16_t
	selmem,
	phmem,
	sample,
	bcntm,
	bcnt0,
	bcnt1;
#endif	// OSC_WIN

#ifdef NOCOMP

// avoid any compensation in modulation
#ifdef REFCOMP
#undef REFCOMP
#endif	// REFCOMP
#ifdef INLCOMP
#undef INLCOMP
#endif	// INLCOMP
#define IU				( 0 )
#define IV				( 0 )
#define IW				( 0 )
#define MAX_VERR		((int16_t)(mul_w(vbus, ((int16_t)(NORMVAL * (DEADTIM / (halfper * 2.0))))) >> SH_NORMVAL))

#else	// NOCOMP

// modulation compensation based on currents
#ifdef REFCOMP
int16_t
	iar,				// reference a current [internal current unit]
	ibr,				// reference b current [internal current unit]
	iur,				// iu reference [int]
	ivr,				// iv reference [int]
	iwr;				// iw reference [int]
#define IU				( iur )
#define IV				( ivr )
#define IW				( iwr )
#else	// REFCOMP
#define IU				( ium )
#define IV				( ivm )
#define IW				( iwm )
#endif	// REFCOMP

// inverter errors compensation
int16_t
	tab_inlx[2];

#ifndef INLCOMP

#define MAX_VERR		((int16_t)(mul_w(vbus, tab_inlx[1]) >> SH_NORMVAL))
//#define MC_INLX( X )	(mul_w(vbus, MC_TabCal(( X ), T_INL_DIN_INT, ((int16_t *)tab_inlx), 1)) >> SH_NORMVAL)
#define MC_INLX( X )	(mul_w(vbus, MC_INLXTabCal( X )) >> SH_NORMVAL)
#define MC_INL( X )		(( X ) >= 0? MC_INLX( X ): (-(MC_INLX(-( X )))))

#else	// INLCOMP

#define INL_DT			((uint16_t)(MTU_FRE_HZ * (INL_DT_NS / 1000000000.0)))
const int16_t tab_inlk[T_INL_INDMAX + 1]=
{
	0,
	((int16_t)(KV * INL_01_VOL)),
	((int16_t)(KV * INL_02_VOL)),
	((int16_t)(KV * INL_03_VOL)),
	((int16_t)(KV * INL_04_VOL)),
	((int16_t)(KV * INL_05_VOL)),
	((int16_t)(KV * INL_06_VOL)),
	((int16_t)(KV * INL_07_VOL)),
	((int16_t)(KV * INL_08_VOL)),
	((int16_t)(KV * INL_09_VOL)),
	((int16_t)(KV * INL_10_VOL)),
	((int16_t)(KV * INL_11_VOL)),
	((int16_t)(KV * INL_12_VOL)),
	((int16_t)(KV * INL_13_VOL)),
	((int16_t)(KV * INL_14_VOL)),
	((int16_t)(KV * INL_15_VOL)),
	((int16_t)(KV * INL_16_VOL)),
	((int16_t)(KV * INL_17_VOL)),
	((int16_t)(KV * INL_18_VOL)),
	((int16_t)(KV * INL_19_VOL)),
	((int16_t)(KV * INL_20_VOL)),
	((int16_t)(KV * INL_21_VOL)),
	((int16_t)(KV * INL_22_VOL)),
	((int16_t)(KV * INL_23_VOL)),
	((int16_t)(KV * INL_24_VOL)),
	((int16_t)(KV * INL_25_VOL)),
	((int16_t)(KV * INL_26_VOL)),
	((int16_t)(KV * INL_27_VOL)),
	((int16_t)(KV * INL_28_VOL)),
	((int16_t)(KV * INL_29_VOL)),
	((int16_t)(KV * INL_30_VOL)),
	((int16_t)(KV * INL_31_VOL))
};
#define MAX_VERR		((int16_t)(tab_inlk[T_INL_INDMAX_U] + (mul_w(vbus, tab_inlx[1]) >> SH_NORMVAL)))
//#define MC_INLX( X )	(mul_w(vbus, MC_TabCal(( X ), T_INL_DIN_INT, ((int16_t *)tab_inlx), 1)) >> SH_NORMVAL)
#define MC_INLX( X )	(mul_w(vbus, MC_INLXTabCal( X )) >> SH_NORMVAL)
//#define MC_INLK( X )	(MC_TabCal(( X ), T_INL_DIN_INT, ((int16_t *)tab_inlk), T_INL_INDMAX_U))
#define MC_INLK( X )	(MC_INLKTabCal( X ))
#define MC_INLU( X )	(MC_INLX( X ) + MC_INLK( X ))
#define MC_INL( X )		(( X ) >= 0? MC_INLU( X ): (-(MC_INLU(-( X )))))

#endif	// INLCOMP

#endif	// NOCOMP

#ifdef MACRO_DEBUG
float32_t
	max_voltage_vol,
	base_voltage_volt,
	base_current_amp,
	base_speed_r_s,
	base_impedance_ohm,
	base_inductance_hen,
	base_flux_web,
	kv,
	ki,
	kr,
	kl,
	kf,
	ko2,
	krpmo,
	korpm,
	kadif,
	kadvf;
int32_t
	deadtim;
void macro_test(void)
{
	max_voltage_vol = MAX_VOLTAGE_VOL;
	base_voltage_volt = BASE_VOLTAGE_VOL;
	base_current_amp = BASE_CURRENT_AMP;
	base_speed_r_s = BASE_SPEED_R_S;
	base_impedance_ohm = BASE_IMPEDANCE_OHM;
	base_inductance_hen = BASE_INDUCTANCE_HEN;
	base_flux_web = BASE_FLUX_WEB;
	kv = KV;
	ki = KI;
	kr = KR;
	kl = KL;
	kf = KF;
	ko2 = KO2;
	krpmo = KRPMO;
	korpm = KORPM;
	kadif = KADIF;
	kadvf = KADVF;
}
#endif	// MACRO_DEBUG

// sampling frequency related constants calculation
/******************************************************************************
* Name:			MC_SetPwmConst
* Description:	calculation of constants related to sampling (and pwm) freq.
* Arguments:	desired sampling frequency in Hz sam_fre_Hz_req
*				desired pwm_freq/sam_freq ratio fre_ratio_req
* Return Value:	0 all OK, an error code if something wrong (clamp
*				performed because limits exceeded)
******************************************************************************/
uint16_t MC_SetPwmConst(uint16_t sam_fre_Hz_req, uint16_t fre_ratio_req)
{
	uint16_t retcode;

	retcode = 0;
	if(sam_fre_Hz_req < MIN_SAM_FRE_HZ)
	{
		sam_fre_Hz = MIN_SAM_FRE_HZ;
		retcode += 1;
	}
	else if(sam_fre_Hz_req > MAX_SAM_FRE_HZ)
	{
		sam_fre_Hz = MAX_SAM_FRE_HZ;
		retcode += 3;
	}
	else
	{
		sam_fre_Hz = sam_fre_Hz_req;
	}
	if(fre_ratio_req < MIN_FRE_RATIO)
	{
		fre_ratio = MIN_FRE_RATIO;
		retcode += 5;
	}
	else if(fre_ratio_req > MAX_FRE_RATIO)
	{
		fre_ratio = MAX_FRE_RATIO;
		retcode += 7;
	}
	else
	{
		fre_ratio = fre_ratio_req;
	}

	if(1 == fre_ratio)
	{
		int_skip = 0;
	}
	else
	{
		int_skip = fre_ratio + 7;	// 8 (ena bit = 1) + (fre_ratio - 1) (number of skipped int)
	}

	pwm_fre_Hz = fre_ratio * sam_fre_Hz;
	pwm_halfper_int = ((uint16_t)(0.5 * MTU_FRE_HZ / pwm_fre_Hz));	// half pwm period in MTU ticks (int)
	pwm_per_int = pwm_halfper_int << 1;								// pwm period in MTU ticks (int)
	sam_per_int = fre_ratio * pwm_per_int;							// sampling period in MTU ticks (int)
	pwm_fre_Hz = ((float32_t)(MTU_FRE_HZ / pwm_per_int));			// true pwm frequency in Hz (float)
	//pwm_per_sec = ((float32_t)(pwm_per_int / MTU_FRE_HZ));		// true pwm period in sec (float)
	sam_fre_Hz = ((float32_t)(MTU_FRE_HZ / sam_per_int));			// true sampling frequency in Hz (float)
	//sam_per_sec = ((float32_t)(sam_per_int / MTU_FRE_HZ));		// true sampling period in sec (float)
	num_int = ((uint16_t)(sam_fre_Hz / MLOOP_FRE_HZ));				// number of sampling periods in one main loop (int)
	mloop_fre_Hz = sam_fre_Hz / num_int;							// true main loop frequency in Hz (float)
	//mloop_per_sec = num_int / sam_fre_Hz;							// true main loop period in sec (float)
	sfkHz1024 = ((uint16_t)(sam_fre_Hz * 1.024f));					// 1024*sam_fre_kHz (int)

	//halfper = pwm_halfper_int;

	halfper_NRM = ((((int32_t)halfper) * ((int32_t)NORMVAL)) / 32767);	// useful in modulation calc
	halfper_amp15 = (((int32_t)halfper) * 32768);					// useful in modulation calc
	halfperdead = halfper + DEADTIM;								// half pwm period plus deadtimes in MTU ticks (int)
	centre_pos = halfper - ((uint16_t)((0.005 * CEN_POS_K) * (halfper - DMIN)));	// desired star centre position
	delmaxp = halfper - DMIN;										// max delta between max duty and min duty in MTU ticks (int), when current is >=0
	delmaxn = halfperdead - DMIN;									// max delta between max duty and min duty in MTU ticks (int), when current is <0
	kvout = ((uint16_t)((NORMVALF * delmaxp) / halfper));			// output voltage reduction ratio (int)
	//kt = sam_fre_Hz;

	ko1 = ((float32_t)(NORMVALF * KAF / kt));						// conversion constant in speed calculation
	ko1_2 = ((int16_t)(ko1 / KO2));									// conversion constant in speed calculation

	McrpLib_SetBaseValues(sam_fre_Hz, BASE_VOLTAGE_VOL, BASE_CURRENT_AMP, BASE_SPEED_R_S);	// setting of base values in library

#ifndef NOCOMP
	tab_inlx[ 0] = 0;
#ifndef INLCOMP
	tab_inlx[ 1] = ((int16_t)(NORMVAL * (DEADTIM / (halfper * 2.0))));
#else	// INLCOMP
	tab_inlx[ 1] = ((int16_t)(NORMVAL * (INL_DT / (halfper * 2.0))));
#endif	// INLCOMP
#endif	// NOCOMP

	return(retcode);
}

/******************************************************************************
* Name:			MC_Cntms
* Description:	calculation of the number of sampling periods corresponding
*				to a given interval in ms
* Arguments:	interval duration in ms millisec
* Return Value:	corresponding number of sampling periods
******************************************************************************/
uint32_t MC_Cntms(uint16_t millisec)
{
	uint32_t aul;
	aul = mul_w(millisec, sfkHz1024);
	return(aul >> 10);
}
// end of sampling frequency related constants calculation

/******************************************************************************
* Function Name: MC_WaitSync
* Description  : time synchronization control function
* Arguments    : none
* Return Value : 0 if main loop time is elapsed, 1 if not
******************************************************************************/
uint16_t MC_WaitSync(void)
{
	if(cnt_int)
	{
		return(1);
	}
	cnt_int = num_int;
	return(0);
}

int16_t MC_TabCal(int16_t in, int16_t din, int16_t *tab, int16_t imx)
/*******************************************************************************
*	Description:	linear interpolation from a table
*******************************************************************************/
{
	int32_t asl;
	int16_t	ass, rem, ind;

	if(in <= 0)
	{
		return(tab[0]);
	}
	{
		//ind = in / din;
		//rem = in % din;
		div_t res;
		res = div(in, din);
		ind = res.quot;
		rem = res.rem;
	}
	if(ind >= imx)
	{
		return(tab[imx]);
	}
	asl = mul_w(rem, (tab[ind + 1] - tab[ind]));
	ass = div_w(asl, din);
	ass += tab[ind];
	return(ass);
}

#ifndef NOCOMP
int16_t MC_INLKTabCal(int16_t in)
/*******************************************************************************
*	Description:	linear interpolation from a table
*******************************************************************************/
{
	int32_t asl;
	int16_t	ass, rem, ind;

	if(in <= 0)
	{
		return(0);
	}
	if(in >= T_INL_INMAX_U)
	{
		return(tab_inlk[T_INL_INDMAX_U]);
	}
	{
		div_t res;
		res = div(in, T_INL_DIN_INT);
		ind = res.quot;
		rem = res.rem;
	}
	asl = mul_w(rem, (tab_inlk[ind + 1] - tab_inlk[ind]));
	ass = div_w(asl, T_INL_DIN_INT);
	ass += tab_inlk[ind];
	return(ass);
}

#ifdef INLCOMP
int16_t MC_INLXTabCal(int16_t in)
/*******************************************************************************
*	Description:	linear interpolation from a table
*******************************************************************************/
{
	int32_t asl;
	int16_t	ass;

	if(in <= 0)
	{
		return(0);
	}
	if(in >= T_INL_DIN_INT)
	{
		return(tab_inlx[1]);
	}
	asl = mul_w(in, tab_inlx[1]);
	ass = div_w(asl, T_INL_DIN_INT);
	return(ass);
}
#endif	// INLCOMP
#endif	// NOCOMP

// debug PWM outputs
#define DBPWM_SH		( 8 )
#define DBPWM_PER		(1 << DBPWM_SH)	// 32MHz/256=125kHz

/******************************************************************************
* Function Name: MC_StartDBPWM
* Description  : set and start debug pwm outputs
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartDBPWM(void)
{
// PIN34 PA1 MTIOC0B OUT ANOUT3
	// wake up module before accessing registers
	//WAKEUP_MTU
	//DELAY1MS(1);

	// stop channel 0, 2 and clear timers
	MTU.TSTR.BIT.CST0 = 0;
	MTU0.TCNT = 0;

	// select pwm mode 2 for MTU0
	MTU0.TMDR.BYTE = 0x03;
	//MTU0.TMDR.BIT.BFE = 0;	//
	//MTU0.TMDR.BIT.BFB = 0;	// no buffer
	//MTU0.TMDR.BIT.BFA = 0;	// no buffer
	//MTU0.TMDR.BIT.MD = 3;		// select pwm mode 2

	// select channel 0 for counter clock, select TGRA0 as period register
	MTU0.TCR.BYTE = 0x20;
	//MTU0.TCR.BIT.CCLR = 1;	// counter cleared by TGRA0 compare match
	//MTU0.TCR.BIT.CKEG = 0;	// count at rising edge
	//MTU0.TCR.BIT.TPSC = 0;	// no prescaler

	// select duty register (TIOC0B)
	//MTU0.TIORH.BYTE = 0x20;
	MTU0.TIORH.BIT.IOB = 2;		// initial output 0, output 1 at compare match (no buffer possible)
	//MTU0.TIORH.BIT.IOA = 0;
	//MTU0.TIORL.BYTE = 0x00;
	//MTU0.TIORL.BIT.IOD = 0;
	//MTU0.TIORL.BIT.IOC = 0;

	// set period in TGRA0, duty in TGRB0
	MTU0.TGRA = DBPWM_PER;		// period
	MTU0.TGRB = DBPWM_PER + 2;	// no compare match-> duty=0

	// pin assignment to the peripheral
	MPC.PWPR.BYTE = 0x00;		// enable writing BIT6
	MPC.PWPR.BYTE = 0x40;		// enable writing MPC registers
	MPC.PA1PFS.BYTE = 0x01;		// PIN34 PA1 MTIOC0B OUT ANOUT3
	MPC.PWPR.BYTE = 0x80;		// disable writing MPC registers

	// port mode register setting
	PORTA.PMR.BIT.B1 = 1;		// PIN34 PA1 MTIOC0B OUT ANOUT3

	// start timer
	MTU.TSTR.BIT.CST0 = 1;		// start timer 0
}

/******************************************************************************
* Function Name: MC_Set_PWM_TST
* Description  : set debug pwm output duty (no buffered operation!)
*				 PA1 (Pin34) is MTU out TIOC0B: PWM_TST
* Arguments    : duty cycle value, 0=min, 65535=max
* Return Value : none
******************************************************************************/
//void MC_Set_PWM_TST(uint16_t val)
//{	// PIN34 PA1 MTIOC0B OUT ANOUT3
//	MTU0.TGRB = DBPWM_PER - (1 + (val >> (16 - DBPWM_SH)));
//}
#define MC_Set_PWM_TST( X )	{MTU0.TGRB = DBPWM_PER - (1 + (((uint16_t)( X )) >> (16 - DBPWM_SH)));}
// end of debug PWM output

/******************************************************************************
* Function Name: MC_SetOff
* Description  : current offsets measurement
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_SetOff(void)
{
	uint32_t off;
	uint16_t i;

	// wake up modules before accessing registers
	WAKEUP_S12AD
	DELAY1MS(1)
	S12AD.ADCSR.BIT.ADIE = 0U;		// Disable S12ADI0 interrupt
	IR(S12AD, S12ADI0) = 0U;		// Clear S12ADI0 interrupt flag

	S12AD.ADCSR.WORD = 0x0400;
	//S12AD.ADCSR.BIT.ADST = 0;		// B15
	//S12AD.ADCSR.BIT.ADCS = 0;		// B14, 13
	//S12AD.ADCSR.BIT.ADIE = 0;		// B12
	//S12AD.ADCSR.BIT.ADHSC = 1;	// B10
	//S12AD.ADCSR.BIT.TRGE = 0;		// B9
	//S12AD.ADCSR.BIT.EXTRG = 0;	// B8
	//S12AD.ADCSR.BIT.DBLE = 0;		// B7
	//S12AD.ADCSR.BIT.GBADIE = 0;	// B6
	//S12AD.ADCSR.BIT.DBLANS = 0;	// B4, .., B0

	S12AD.ADCER.WORD = 0x0000;
	//S12AD.ADANSA.WORD = 0x0000;
	//S12AD.ADANSB.WORD = 0x0000;
	//S12AD.ADADS.WORD = 0x0000;
	//S12AD.ADADC.BYTE = 0x0000;
	//S12AD.ADEXICR.WORD = 0x0000;

    S12AD.ADSSTR0 = SAM_STA;
	S12AD.ADSSTR1 = SAM_STA;
	S12AD.ADSSTR2 = SAM_STA;
	S12AD.ADSSTR6 = SAM_STA;

	// pins assignment to the peripheral
	MPC.PWPR.BYTE = 0x00;		// enable writing BIT6
	MPC.PWPR.BYTE = 0x40;		// enable writing MPC registers
	//MPC.PJ6PFS.BYTE = 0x00;		// AD_REF_HIGH_AVCC0
	//MPC.PJ7PFS.BYTE = 0x00;		// AD_REF_LOW_AVSS0
	MPC.P40PFS.BYTE = 0x80;		// PIN45 P40 AN0 IN  IU
	//PORT4.PMR.BIT.B0 = 0;			// PIN45 P40 AN0 IN  IU
	//PORT4.PDR.BIT.B0 = 0;			// PIN45 P40 AN0 IN  IU
	MPC.P41PFS.BYTE = 0x80;		// PIN43 P41 AN1 IN  IV
	//PORT4.PMR.BIT.B1 = 0;			// PIN43 P41 AN1 IN  IV
	//PORT4.PDR.BIT.B1 = 0;			// PIN43 P41 AN1 IN  IV
	MPC.P42PFS.BYTE = 0x80;		// PIN42 P42 AN2 IN  IW
	//PORT4.PMR.BIT.B2 = 0;			// PIN42 P42 AN2 IN  IW
	//PORT4.PDR.BIT.B2 = 0;			// PIN42 P42 AN2 IN  IW
	MPC.P46PFS.BYTE = 0x80;		// PIN40 P46 AN6 IN  VB
	//PORT4.PMR.BIT.B6 = 0;			// PIN41 P46 AN6 IN  VB
	//PORT4.PDR.BIT.B6 = 0;			// PIN41 P46 AN6 IN  VB
	MPC.PE1PFS.BYTE = 0x80;		// PIN38 PE1 AN9 IN  TMP
	//PORTE.PMR.BIT.B1 = 0;			// PIN38 PE1 AN9 IN  TMP
	//PORTE.PDR.BIT.B1 = 0;			// PIN38 PE1 AN9 IN  TMP
	MPC.PE2PFS.BYTE = 0x80;		// PIN37 PE2 AN10 IN  REF
	//PORTE.PMR.BIT.B2 = 0;			// PIN37 PE2 AN10 IN  AUX
	//PORTE.PDR.BIT.B2 = 0;			// PIN37 PE2 AN10 IN  AUX
	MPC.PWPR.BYTE = 0x80;		// disable writing MPC registers

	// channel 0
	SEL_IU_AD
	for(i = 0, off = 0; i < 32; i++)
	{
		S12AD_START
        WAIT_S12AD
		off += AN0_IU;
		DELAY1MS(1)
	}
	ium_off = off >> 5;
	ium_ofm = mulu_w(ium_off, G_OFF_LPKT);	// ium_ofm = ium_off; ium_ofm <<= SH_OFF_LPKT;

	// channel 1
	SEL_IV_AD
	for(i = 0, off = 0; i < 32; i++)
	{
		S12AD_START
        WAIT_S12AD
		off += AN1_IV;
		DELAY1MS(1)
	}
	ivm_off = off >> 5;
	ivm_ofm = mulu_w(ivm_off, G_OFF_LPKT);	// ivm_ofm = ivm_off; ivm_ofm <<= SH_OFF_LPKT;

	// channel 2
	SEL_IW_AD
	for(i = 0, off = 0; i < 32; i++)
	{
		S12AD_START
        WAIT_S12AD
		off += AN2_IW;
		DELAY1MS(1)
	}
	iwm_off = off >> 5;
	iwm_ofm = mulu_w(iwm_off, G_OFF_LPKT);	// iwm_ofm = iwm_off; iwm_ofm <<= SH_OFF_LPKT;

}

/******************************************************************************
* Function Name: MC_ADMan
* Description  : management of A/D conversions for currents
* Arguments    : none (global variable cr_status is used to determinate the
*				 situation, the A/D registers are read for the currents)
* Return Value : none (global variables ium, ivm, iwm are modified)
******************************************************************************/
static void MC_ADMan(void)
{
	int32_t asl;

	switch(cr_status)
	{
		case CR_UV:
			asl = mul_w(AN0_IU - ium_off, KADI);
			ium = asl >> SH_KADI;
			asl = mul_w(AN1_IV - ivm_off, KADI);
			ivm = asl >> SH_KADI;
			iwm = -ium - ivm;
			break;
		case CR_VW:
			asl = mul_w(AN1_IV - ivm_off, KADI);
			ivm = asl >> SH_KADI;
			asl = mul_w(AN2_IW - iwm_off, KADI);
			iwm = asl >> SH_KADI;
			ium = -ivm - iwm;
			break;
		case CR_WU:
			asl = mul_w(AN2_IW - iwm_off, KADI);
			iwm = asl >> SH_KADI;
			asl = mul_w(AN0_IU - ium_off, KADI);
			ium = asl >> SH_KADI;
			ivm = -ium - iwm;
			break;
		default:
			ium = 0;
			ivm = 0;
			iwm = 0;
			break;
	}
}	/* end of function MC_ADMan() */

/******************************************************************************
* Function Name: MC_IniPWM
* Description  : pwm initialization
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_IniPWM(void)
{

	// wake up module before accessing registers
	WAKEUP_MTU
	DELAY1MS(1);

	// MTU general settings
	MTU.TRWER.BIT.RWE = 1;		// MTU registers writing enable
	MTU.TSTR.BIT.CST3 = 0;		// stop timer 3
	MTU.TSTR.BIT.CST4 = 0;		// stop timer 4

	MTU3.TCR.BYTE = 0x00;		//
	//MTU3.TCR.BIT.CCLR = 0;	// synchronous clearing
	//MTU3.TCR.BIT.CKEG = 0;	// count at rising edge
	//MTU3.TCR.BIT.TPSC = 0;	// no peripheral clock frequency divider
	MTU4.TCR.BYTE = 0x00;		//
	//MTU4.TCR.BIT.CCLR = 0;	// synchronous clearing
	//MTU4.TCR.BIT.CKEG = 0;	// count at rising edge
	//MTU4.TCR.BIT.TPSC = 0;	// no peripheral clock frequency divider

	// registers and buffers setting
	MTU3.TCNT = DEADTIM;		// deadtime in TCNT3
	MTU4.TCNT = 0;				// 0 in TCNT4
	MTU.TDDR = DEADTIM;			// deadtime in TDDR
	MTU.TCDR = halfper;			// half pwm period in TCDR (note: MUST be > (TDDR*2+2)
	MTU.TCBR = halfper;			// buffer for TCDR
	MTU3.TGRA = halfperdead;	// halfper + deadtime in TGRA3
	MTU3.TGRB = halfperdead;	// duty U (0%, low arms always on)
	MTU4.TGRA = halfperdead;	// duty V (0%, low arms always on)
	MTU4.TGRB = halfperdead;	// duty W (0%, low arms always on)
	MTU3.TGRC = halfperdead;	// buffer for TGRA3
	MTU3.TGRD = halfperdead;	// buffer for TGRB3
	MTU4.TGRC = halfperdead;	// buffer for TGRA4
	MTU4.TGRD = halfperdead;	// buffer for TGRB4 (LAST TO BE WRITTEN!)

	// pwm frequency output enable and active levels setting
	MTU.TOCR1.BIT.PSYE = 0;		// disable output of pwm frequency
	MTU.TOCR1.BIT.OLSN = 1;		// select high-active output level
	MTU.TOCR1.BIT.OLSP = 1;		// select high-active output level

	// buffer setting
	MTU3.TMDR.BIT.BFB = 1;		// TGRD3 is a buffer for TGRB3
	MTU3.TMDR.BIT.BFA = 1;		// TGRC3 is a buffer for TGRA3
	//MTU4.TMDR.BIT.BFB = 1;	// TGRD4 is a buffer for TGRB4
	//MTU4.TMDR.BIT.BFA = 1;	// TGRC4 is a buffer for TGRA4

	// complementary pwm mode setting
	MTU3.TMDR.BIT.MD = 0x0D;	// complementary PWM mode, transmit at crest
	//MTU3.TMDR.BIT.MD = 0x0E;	// complementary PWM mode, transmit at trough
	//MTU3.TMDR.BIT.MD = 0x0F;	// complementary PWM mode, transmit at crest and trough
	//MTU4.TMDR.BIT.MD = 0;		// do not set here, but only in MTU3.TMDR

	// output disable
	MTU.TOER.BYTE = 0xC0;		// all outputs disabled (at inactive level)
	//MTU.TOER.BYTE = 0xFF;		// all outputs enabled

	// pins assignment to the peripheral
	// NOW the pins are configured as I/O ports, outputs with output value 0 (from hwsetup)

	// enable writing
	MPC.PWPR.BYTE = 0x00;		// enable writing BIT6
	MPC.PWPR.BYTE = 0x40;		// enable writing MPC registers

	// Set MTIOC3B pin23 PWM_UH
    MPC.PC5PFS.BYTE = 0x01;
	//PORTC.PODR.BIT.B5 = 0;	// PIN23 PC5 MTIOC3B OUT PWM_UH
	//PORTC.PDR.BIT.B5 = 1;		// PIN23 PC5 MTIOC3B OUT PWM_UH
	PORTC.PMR.BIT.B5 = 1;		// PIN23 PC5 MTIOC3B OUT PWM_UH

    // Set MTIOC3D pin24 PWM_UL
    MPC.PC4PFS.BYTE = 0x01;
	//PORTC.PODR.BIT.B4 = 0;	// PIN24 PC4 MTIOC3D OUT PWM_UL
	//PORTC.PDR.BIT.B4 = 1;		// PIN24 PC4 MTIOC3D OUT PWM_UL
	PORTC.PMR.BIT.B4 = 1;		// PIN24 PC4 MTIOC3D OUT PWM_UL

    // Set MTIOC4A pin26 PWM_VH
    MPC.PB3PFS.BYTE = 0x02;
	//PORTB.PODR.BIT.B3 = 0;	// PIN26 PB3 MTIOC4A OUT PWM_VH
	//PORTB.PDR.BIT.B3 = 1;		// PIN26 PB3 MTIOC4A OUT PWM_VH
	PORTB.PMR.BIT.B3 = 1;		// PIN26 PB3 MTIOC4A OUT PWM_VH

    // Set MTIOC4C pin27 PWM_VL
    MPC.PB1PFS.BYTE = 0x02;
	//PORTB.PODR.BIT.B1 = 0;	// PIN27 PB1 MTIOC4C OUT PWM_VL
	//PORTB.PDR.BIT.B1 = 1;		// PIN27 PB1 MTIOC4C OUT PWM_VL
	PORTB.PMR.BIT.B1 = 1;		// PIN27 PB1 MTIOC4C OUT PWM_VL

    // Set MTIOC4B pin36 PWM_WH
    MPC.PE3PFS.BYTE = 0x01;
	//PORTE.PODR.BIT.B3 = 0;	// PIN36 PE3 MTIOC4B OUT PWM_WH
	//PORTE.PDR.BIT.B3 = 1;		// PIN36 PE3 MTIOC4B OUT PWM_WH
	PORTE.PMR.BIT.B3 = 1;		// PIN36 PE3 MTIOC4B OUT PWM_WH

    // Set MTIOC4D pin35 PWM_WL
    MPC.PE4PFS.BYTE = 0x01;
	//PORTE.PODR.BIT.B4 = 0;	// PIN35 PE4 MTIOC4D OUT PWM_WL
	//PORTE.PDR.BIT.B4 = 1;		// PIN35 PE4 MTIOC4D OUT PWM_WL
	PORTE.PMR.BIT.B4 = 1;		// PIN35 PE4 MTIOC4D OUT PWM_WL

	// disable writing
	MPC.PWPR.BYTE = 0x80;		// disable writing MPC registers

	// POE init for pwm pins protection
	POE.ICSR1.WORD = 0x0000;	// pin POE1 is used, falling edge 
	//POE.ICSR1.BIT.POE1M = 0;	// POE1 Mode	0 -> Falling edge (modif. only once)
	//POE.ICSR1.BIT.PIE1 = 0;	// POE Port 1 interrupt enable (POE 0, 1, 2)
	//POE.ICSR1.BIT.POE1F = 0;	// POE1 Flag
	//INTC.IPRF.BIT._MTU_POE = 5;	// POE
	POE.OCSR1.WORD = 0x20;		// POE output level control/status register
	//POE.OCSR1.BIT.OCE1 = 1;	// POE output 1 short circuit high impedance control (modif. only once)
	//POE.OCSR1.BIT.OIE1 = 0;	// POE output 1 short circuit interrupt enable
	//POE.OCSR1.BIT.OSF1 = 0;	// POE output 1 short circuit Flag
	POE.POECR2.BIT.P1CZEA = 1;	// PE9-PE11		UH-UL
	POE.POECR2.BIT.P2CZEA = 1;	// PE12-PE14	VH-VL
	POE.POECR2.BIT.P3CZEA = 1;	// PE13-PE15	WH-WL
	//POE.SPOER.BIT.MTUCH34HIZ = 0;	// MTU ch 3, 4 not in HIZ

	// pin assignment to the peripheral
	MPC.PWPR.BYTE = 0x00;		// enable writing BIT6
	MPC.PWPR.BYTE = 0x40;		// enable writing MPC registers
	//MPC.PC3PFS.BYTE = 0x07;	// PIN25 PB5 (PC3) POE0 IN  OVC
	MPC.PB5PFS.BYTE = 0x07;		// PIN25 PB5 (PC3) POE0 IN  OVC
	MPC.PWPR.BYTE = 0x80;		// disable writing MPC registers

	// port mode register setting
	PORTB.PMR.BIT.B5 = 1;		// PIN25 PB5 (PC3) POE0 IN  OVC

}

/******************************************************************************
* Function Name: MC_StopOut
* Description  : disable pwm outputs
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StopOut(void)
{
#ifdef DEF_LOW_ON
	if(FAULT_ALL == UIF_R.var.all)
	{
		MTU.TOER.BYTE = 0xC0;	// all outputs disabled (at inactive level)
	}
#else	// DEF_LOW_ON
	MTU.TOER.BYTE = 0xC0;		// all outputs disabled (at inactive level)
#endif	// DEF_LOW_ON
	RES_COM_ON
	RES_COM_REQ
}

/******************************************************************************
* Function Name: MC_StartOut
* Description  : enable pwm outputs and insert a loop for bootstrap
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartOut(void)
{
	if(0 == UIF_R.var.all)
	{
		if(COM_OFF)
		{
			if(COM_REQ)
			{
				SET_COM_ON
			}
			else
			{
				SET_COM_REQ
				MTU.TOER.BYTE = 0xFF;	// all outputs enabled
			}
		}
	}
	else
	{
		MC_StopOut();
	}
}

/******************************************************************************
* Function Name: MC_EnablePOE
* Description  : POE interrupt enable
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_EnablePOE(void)
{
	if(0 != POE.ICSR1.BIT.POE1F)
	{
		POE.ICSR1.BIT.POE1F = 0;	// reset flag
	}
	POE.ICSR1.BIT.PIE1 = 1;			// interrupt request enable
	if(0 != POE.OCSR1.BIT.OSF1)
	{
		POE.OCSR1.BIT.OSF1 = 0;		// reset flag
	}
	POE.OCSR1.BIT.OIE1 = 1;			// interrupt request enable
	IPR(POE, OEI1) = OVC_INT_P;		// interrupt priority
	IR(POE, OEI1) = 0;				// reset interrupt request
	IEN(POE, OEI1) = 1;				// interrupt enable
}

/******************************************************************************
* Function Name: MC_StartPWM
* Description  : start pwm and main interrupt (but not the outputs)
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartPWM(void)
{
	// interrupt skipping (Timer 4 underflow -> trough)
	MTU.TITCR.BYTE = 0x00;			// Timer Interrupt Skipping Set Register 1A (Reset val 0x00)
	//MTU.TITCR.BIT.T3AEN = 0;		// B7
	//MTU.TITCR.BIT.T3ACOR = 0;		// B6, B5, B4
	//MTU.TITCR.BIT.T4VEN = 0;		// B3
	//MTU.TITCR.BIT.T4VCOR = 0;		// B2,B1, B0
	MTU.TITCR.BYTE |= int_skip;

	// A/D trigger configuration
	//MTU4.TADCOBRA = CURAD_T;
	MTU4.TADCORA = CURAD_T;
	MTU4.TADCR.BIT.BF = 0;			// B15, B14 Buffer Transfer Timing Select: 0 no, 1 crest, 2 trough, 3 both
	MTU4.TADCR.BIT.DT4AE = 1;		// B6 Down-Count TRG4AN Enable
	if(0 != MTU.TITCR.BIT.T4VEN)
	{
		MTU4.TADCR.BIT.ITA4VE = 1;	// B2 TCIV4 interrupt skipping is linked
	}
	else
	{
		MTU4.TADCR.BIT.ITA4VE = 0;	// B2 TCIV4 interrupt skipping is not linked
	}
	S12AD.ADSTRGR.BIT.TRSA = 6;		// select TRGA4N (MTU4.TCNT and MTU4.TADCORA compare match) as trigger source
	S12AD.ADCSR.BIT.TRGE = 1;		// enable trigger

/*
	// A/D trigger configuration
	MTU4.TIER.BIT.TTGE2 = 1;		// enable A/D trigger at TCNT4 underflow (trough)
	S12AD.ADSTRGR.BIT.TRSA = 3;		// select TRGAN (trough) as trigger source
	S12AD.ADCSR.BIT.TRGE = 1;		// enable trigger
*/
/*
	// interrupt setup (TCNT4 underflow, trough)
	MTU4.TSR.BIT.TCFD = 0;			// clear flag of TCNT4 underflow (trough)
	MTU4.TIER.BIT.TCIEV = 1;		// enable interrupt request on TCFV flag
	IPR(MTU4, TCIV4) = MC_INT_P;	// interrupt priority
	IR(MTU4, TCIV4) = 0;			// reset interrupt request
	IEN(MTU4, TCIV4) = 1;			// interrupt enable
*/

	// channel select
	SEL_CUR_AD

	// interrupt setup
	S12AD.ADCSR.WORD = 0x1600; 		// ADIE, ADHSC, TRGE are 1
	IPR(S12AD, S12ADI0) = MC_INT_P;	// interrupt priority (S12ADI0)
	IR(S12AD, S12ADI0) = 0;			// reset interrupt request
	IEN(S12AD, S12ADI0) = 1;		// enable interrupt

	// timer start
	MTU.TSTR.BYTE |= 0xC0;			// simultaneously start timer 3 and 4

	// POE interrupt enable
	MC_EnablePOE();

	// stop outputs for safety
	MC_StopOut();

}

//debug int16_t	starcentre;
/******************************************************************************
* Name:			MC_PWMGen
* Description:	output duties calculation, and current reading status cr_status
*				determination
* Arguments:	none
* Return value:	none
* Note:			global variables vuc, vvc and vwc are the requested voltages,
*				and they are both inputs and outputs for the routine;
*				global variables vbus, IU, IV, IW are used by the routine but
*				not modified;
*				global variables duty_u, duty_v, duty_w and cr_status are the
*				outputs of the routine;
******************************************************************************/
static void MC_PWMGen(void)
{
	int32_t	asl;
#ifndef NOCOMP
	int16_t	eu, ev, ew;
#endif	// NOCOMP
	int16_t	*amin, *amed, *amax;	// addresses of min duty, med duty, max duty
	int16_t	dmmin, dmmed, dmmax;	// delta max for min duty, med duty, max duty
	int16_t	ass, ms, hmed, hmax;

	if(VBMIN > vbus)
	{
		duty_u = duty_v = duty_w = halfperdead;
		vuc = vvc = vwc = 0;
		cr_status = CR_NO;
		return;
	}

#ifndef NOCOMP
	// errors calculation and desired output voltages pre-compensation
	eu = MC_INL(IU);
	ev = MC_INL(IV);
	ew = MC_INL(IW);
	vuc += eu;
	vvc += ev;
	vwc += ew;
#endif	// NOCOMP

	// duty-cycles calculation
#define n_shft	ms					// saves a variable
	ass = halfper_NRM;
	n_shft = SH_NORMVAL;
	while(ass > vbus)
	{
		ass >>= 1;
		n_shft -= 1;
	}
	asl = halfper;
	asl <<= n_shft;
	ass = div_w(asl, vbus);
	asl = mul_w((vbus - vuc), ass);
	duty_u = asl >> n_shft;
	asl = mul_w((vbus - vvc), ass);
	duty_v = asl >> n_shft;
	asl = mul_w((vbus - vwc), ass);
	duty_w = asl >> n_shft;

	// finding duties order and limits
	if(duty_u >= duty_v)
	{
		if(duty_v >= duty_w)
		{
			cr_status = CR_UV;				// w is the min -> we choose iu and iv
			amin = &duty_w;
			dmmin = DELMAX(IW);
			amed = &duty_v;
			dmmed = DELMAX(IV);
			amax = &duty_u;
			dmmax = DELMAX(IU);
		}
		else
		{
			cr_status = CR_WU;				// v is the min -> we choose iw and iu
			amin = &duty_v;
			dmmin = DELMAX(IV);
			if(duty_u >= duty_w)
			{
				amed = &duty_w;
				dmmed = DELMAX(IW);
				amax = &duty_u;
				dmmax = DELMAX(IU);
			}
			else
			{
				amed = &duty_u;
				dmmed = DELMAX(IU);
				amax = &duty_w;
				dmmax = DELMAX(IW);
			}
		}
	}
	else
	{
		if(duty_u >= duty_w)
		{
			cr_status = CR_UV;				// w is the min -> we choose iu and iv
			amin = &duty_w;
			dmmin = DELMAX(IW);
			amed = &duty_u;
			dmmed = DELMAX(IU);
			amax = &duty_v;
			dmmax = DELMAX(IV);
		}
		else
		{
			cr_status = CR_VW;				// u is the min -> we choose iv and iw
			amin = &duty_u;
			dmmin = DELMAX(IU);
			if(duty_v >= duty_w)
			{
				amed = &duty_w;
				dmmed = DELMAX(IW);
				amax = &duty_v;
				dmmax = DELMAX(IV);
			}
			else
			{
				amed = &duty_v;
				dmmed = DELMAX(IV);
				amax = &duty_w;
				dmmax = DELMAX(IW);
			}
		}
	}

	// duties differences calculation
	hmed = (*amed) - (*amin);
	hmax = (*amax) - (*amin);

	// maximum limits control (and saturation)
	if(hmax > dmmax)
	{
		asl = mul_w(hmed, dmmax);
		hmed = div_w(asl, hmax);
		hmax = dmmax;
	}
	if(hmed > dmmed)
	{
		asl = mul_w(hmax, dmmed);
		hmax = div_w(asl, hmed);
		hmed = dmmed;
	}
	
	// possible shift calculation
	ms = dmmax - hmax;
	ass = dmmed - hmed;
	if(ass < ms)
	{
		ms = ass;
	}
	if(dmmin < ms)		// probably not really needed
	{
		ms = dmmin;
	}
	ms += DMIN;			// this is the minimum duty (max voltage) if clamped modulation

// now we have min_duty equal to ms (corresponding to maximum voltage), med_duty equal to (ms + hmed), and max_duty equal to (ms + hmax)
// at least one of the last two is at its maximum value (halfper or halfperdead), that is, the modulation is clamped at the minimum voltage

#ifdef CENTRED
	// shift modification to obtain centred modulation
	ass = hmax + hmed + mul_w(ms, 3);
	asl = mul_w(ass, (NORMVAL / 3));			// (amplified) position of star centre
	ass = (asl >> SH_NORMVAL) - centre_pos;		// distance from star centre and desired centre position
	if(ass > 0)									// we can only decrease the duties (increase the voltages)
	{
		if(ass < (ms - DMIN))					// cannot go under the minimum
		{
			ms -= ass;
		}
		else
		{
			ms = DMIN;
		}
	}
#endif	// CENTRED

	// duties update
	(*amin) = ms;
	(*amed) = ms + hmed;
	(*amax) = ms + hmax;

//debug	ass = hmax + hmed + mul_w(ms, 3);
//debug	asl = mul_w(ass, (NORMVAL / 3));
//debug	starcentre = asl >> SH_NORMVAL;

	// output voltages re-calculation (necessary if any clamp has been executed)
	asl = mul_w(vbus, NORMVAL);
	n_shft = SH_NORMVAL;
	while(asl >= halfper_amp15)
	{
		asl >>= 1;
		n_shft -= 1;
	}
	ass = div_w(asl, halfper);
	asl = mul_w((halfper - duty_u), ass);
	vuc = asl >> n_shft;
	asl = mul_w((halfper - duty_v), ass);
	vvc = asl >> n_shft;
	asl = mul_w((halfper - duty_w), ass);
	vwc = asl >> n_shft;
#ifndef NOCOMP
	vuc -= eu;
	vvc -= ev;
	vwc -= ew;
#endif	// NOCOMP

	// commutations number reducing if possible
#ifndef NOCOMP
	if((halfper == duty_u) && (IU >= 0))
	{
		duty_u = halfperdead;
	}
	if((halfper == duty_v) && (IV >= 0))
	{
		duty_v = halfperdead;
	}
	if((halfper == duty_w) && (IW >= 0))
	{
		duty_w = halfperdead;
	}
#endif	// NOCOMP

}	// end of function MC_PWMGen()

/******************************************************************************
* Function Name: MC_VecComp
* Description  : calculation of the second component of a vector (absolute value)
* Arguments    : vector amplitude amp_vec (greater than zero)
*				 first component fir_comp
* Return Value : second component
* Note         : result is positive or zero
******************************************************************************/
int16_t MC_VecComp(int16_t amp_vec, int16_t fir_comp)
{
	int32_t asl;
	int16_t	ass;

	if(amp_vec <= 0)
	{
		return(0);
	}
	if(0 == fir_comp)
	{
		return(amp_vec);
	}
	if(fir_comp <= -32767)
	{
		//fir_comp = 32767;
		return(0);						// surely >= amp_vec
	}
	else if(fir_comp < 0)
	{
		fir_comp = -fir_comp;
	}
	if(fir_comp >= amp_vec)
	{
		return(0);
	}
	asl = mul_w(fir_comp, NORMSIN);		// asl = fir_comp; asl <<= SH_NORMSIN;
	ass = div_w(asl, amp_vec);
	ass = McrpLib_sinarcos(ass);
	asl = mul_w(ass, amp_vec);
	asl >>= SH_NORMSIN;
	return( (int16_t)asl );
}

/******************************************************************************
* Function Name: MC_IniPar
* Description  : internal constants initialization, using eeprom values
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_IniPar(void)
{
	uint32_t aul;
	uint16_t aus;

	// pwm constants setting, based on eeprom values
	MC_SetPwmConst(SAM_FRE, F_RATIO);
	UIF_R.var.pwmf = pwm_fre_Hz;
	UIF_R.var.sf = sam_fre_Hz;

#ifdef MACRO_DEBUG
	macro_test();
#endif	// MACRO_DEBUG

	// additional features enabling
#ifdef CPI_TUN
	ENA_CURPI_TUN
#endif	// CPI_TUN
#ifdef CPI_AUT
	ENA_CURPI_AUT
#endif	// CPI_AUT
#ifdef	MOT_IDENT
	ENA_AUTO_IDEN
#endif	//	MOT_IDENT
#ifdef OSC_WIN
	ENA_OSCI_WIND
#endif	// OSC_WIN

	// resistance conversion constant normalization
	aul = (uint32_t)KR;
	sh_kr_low = 0;
	while(aul > 32767UL)
	{
		++sh_kr_low;
		aul >>= 1;
	}
	kr_low = aul;

	// inductance conversion constant normalization
	aul = (uint32_t)KL;
	sh_kl_low = 0;
	while(aul > 32767UL)
	{
		++sh_kl_low;
		aul >>= 1;
	}
	kl_low = aul;

	// flux conversion constant normalization
	aul = (uint32_t)KF;
	sh_kf_low = 0;
	while(aul > 32767UL)
	{
		++sh_kf_low;
		aul >>= 1;
	}
	kf_low = aul;

	// polar couples and related constants
	c_poli = (int16_t)C_POLI;
	//krpmocp = mul_w(c_poli, KRPMO);	// omega[int2]=speed[rpm]*krpmocp/NORMVAL
	korpmdcp = divu_w(KORPM, c_poli);	// speed[rpm]=omega[int2]*korpmdcp/NORMVAL

	// rpm max and min
	rpm_max = (int16_t)RPM_MAX;
	rpm_min = (int16_t)RPM_MIN;

	// max angular speed, max speed for trip detection [int2]
	aul = mulu_w(rpm_max, c_poli);
	aul *= KRPMO;			// long_multiply
	max_speed2 = aul >> SH_NORMVAL;
	aus = max_speed2 + (max_speed2 >> 2);	// 125%
	max_speed_trip = mulu_w(aus, ko1_2);

	// min angular speed, and min speed for trip detection [int2]
	aul = mulu_w(rpm_min, c_poli);
	aul *= KRPMO;			// long_multiply
	min_speed2 = aul >> SH_NORMVAL;
	aus = min_speed2 >> 1;					// 50%
	min_speed_trip = mulu_w(aus, ko1_2);

	// startup acceleration time
	st_tim_ep = ST_TIM;
	stup_tim = MC_Cntms(st_tim_ep);
	if(stup_tim > 65535)
	{
		stup_tim = 65535;
	}

	// delta speed every startup step
	om_chg = mulu_w(min_speed2, ko1_2);
	delta_om = divu_w(om_chg, stup_tim);

	// acceleration ramp in internal units
	r_acc_ep = R_ACC;
	aul = r_acc_ep;
	r_acc = divu_w(aul, mloop_fre_Hz);

	// deceleration ramp in internal units
	r_dec_ep = R_DEC;
	aul = r_dec_ep;
	r_dec = divu_w(aul, mloop_fre_Hz);

	// startup current in internal units
	i_start_ep=I_START;
	aul = mulu_w(i_start_ep, KI);
	i_start = divu_w(aul, AMP_RES);
	i_startl = mulu_w(i_start, G_IDRL);
	aul = MC_Cntms(STUP_CUR_RT_MS);
	if(aul > 65535)
	{
		aul = 65535;
	}
	is_rup = divu_w(i_startl, aul);
	aul = MC_Cntms(STUP_CUR_FT_MS);
	if(aul > 65535)
	{
		aul = 65535;
	}
	is_rdw = divu_w(i_startl, aul);

	// maximum current in internal units
	i_max_ep = I_MAX;
	aul = mulu_w(i_max_ep, KI);
	i_max = divu_w(aul, AMP_RES);
#ifdef FLUXWEAK
	id_maxl = mulu_w(i_max, GFW_IDRL);
	aul = MC_Cntms(FW_CUR_RT_MS);
	if(aul > 65535)
	{
		aul = 65535;
	}
	i_del = divu_w(id_maxl, aul);
#endif	// FLUXWEAK
#ifdef MOT_IDENT
	aul = mulu_w(i_max, (HIG_CUR_RATIO * NORMVALF));
	aus = aul >> SH_NORMVAL;
	ial_l1 = mulu_w(aus, G_IDRL);
	aul = mulu_w(i_max, (MID_CUR_RATIO * NORMVALF));
	aus = aul >> SH_NORMVAL;
	ial_l2 = mulu_w(aus, G_IDRL);
	aul = mulu_w(i_max, (LOW_CUR_RATIO * NORMVALF));
	aus = aul >> SH_NORMVAL;
	ial_l3 = mulu_w(aus, G_IDRL);
	aul = MC_Cntms(CUR_RISE_TIME_MS);
	if(aul > 65535)
	{
		aul = 65535;
	}
	ial_r = divu_w(ial_l1, aul);
	ial_l = 0;
#endif	// MOT_IDENT

	// statoric resistence in internal units
	r_sta_ep = R_STA;
	aul = mulu_w(r_sta_ep, kr_low);
	aul <<= sh_kr_low;
	r_sta = divu_w(aul, OHM_RES);

	// synchronous inductance in internal units
	l_syn_ep = L_SYN;
	aul = mulu_w(l_syn_ep, kl_low);
	aul <<= sh_kl_low;
	l_syn = divu_w(aul, HEN_RES);

	// permanent magnets flux in internal units
	pm_flx_ep = PM_FLX;
	aul = mulu_w(pm_flx_ep, kf_low);
	aul <<= sh_kf_low;
	pm_flx = divu_w(aul, WEB_RES);

	// PI gains
	kp_cur = (int16_t)KP_CUR;
	aul = mulu_w(KI_CUR, AMP_KISF);
	if((aul >> 15) >= sam_fre_Hz)
	{
		ki_cur = 32767;
	}
	else
	{
		ki_cur = divu_w(aul, sam_fre_Hz);
	}
	kp_vel = (int16_t)KP_VEL;
	aul = mulu_w(KI_VEL, AMP_KISF);
	if((aul >> 15) >= sam_fre_Hz)
	{
		ki_vel = 32767;
	}
	else
	{
		ki_vel = divu_w(aul, sam_fre_Hz);
	}

	// phase offset
	pha_off_ep = PHA_OFF;
	pha_ofa = mulu_w(pha_off_ep, ((int16_t)(TWOPI / 360.0)));
	pha_off = 0;

	// flux estimation filter time constant tau
	flx_ftau_ep = FLX_FTAU;
	aul = mulu_w(flx_ftau_ep, sam_fre_Hz);
	flx_ftau = divu_w(aul, SEC_RES);

	// flux estimation algorithm parameters setting
	fb_gain = (int16_t)FB_GAIN;
#ifdef EXACT
	McrpLib_FESetPar(r_sta, l_syn, pm_flx, fb_gain);
#else	// EXACT
	McrpLib_FEASetPar(r_sta, l_syn, flx_ftau);
#endif	// EXACT

	// speed ramp state machine
	ramp_status = STOP_0;
	SET_STOPPED
	RES_STARTED

	// phase offset
	pha_off = 0;

	// buffers
	rbuf = ((uint16_t *)(outbuf.ss));
	wbuf = ((uint16_t *)(outbuf1.ss));
	pbuf = rbuf;
	for(aus = 0; aus < OUTBUF_SDIM; aus++)
	{
		rbuf[aus] = 0;
		wbuf[aus] = 0;
	}

#ifdef OSC_WIN
	selmem = 0;
	bcnt0 = 0;
	bcnt1 = 0;
	bcntm = 0;
	phmem = 0;
	sample = 0;
	RES_TRGED
	RES_RBUSY
	RES_CSELD
#endif	// OSC_WIN

	// mode
	SET_NORM_MODE

}

/******************************************************************************
* Function Name: MC_UpdPar
* Description  : internal constants update, if parameters are changed
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_UpdPar(void)
{
	uint32_t aul;
	uint16_t aus;

	// parameters update if changed
	if(	(RPM_MIN != rpm_min) || (RPM_MAX != rpm_max) ||
		(C_POLI != c_poli)   || (ST_TIM != st_tim_ep) )
	{
		rpm_min = (int16_t)RPM_MIN;
		rpm_max = (int16_t)RPM_MAX;
		c_poli = (int16_t)C_POLI;
		st_tim_ep = (int16_t)ST_TIM;
		//krpmocp = mul_w(c_poli, KRPMO);	// omega[int2]=speed[rpm]*krpmocp/NORMVAL
		korpmdcp = divu_w(KORPM, c_poli);	// speed[rpm]=omega[int2]*korpmdcp/NORMVAL
		aul = mulu_w(rpm_max, c_poli);
		aul *= KRPMO;			// long_multiply
		max_speed2 = aul >> SH_NORMVAL;
		aus = max_speed2 + (max_speed2 >> 2);	// 125%
		max_speed_trip = mulu_w(aus, ko1_2);
		aul = mulu_w(rpm_min, c_poli);
		aul *= KRPMO;			// long_multiply
		min_speed2 = aul >> SH_NORMVAL;
		aus = min_speed2 >> 1;					// 50%
		min_speed_trip = mulu_w(aus, ko1_2);
		om_chg = mulu_w(min_speed2, ko1_2);
		stup_tim = MC_Cntms(st_tim_ep);
		if(stup_tim > 65535)
		{
			stup_tim = 65535;
		}
		delta_om = divu_w(om_chg, stup_tim);
	}
	if(R_ACC != r_acc_ep)
	{
		r_acc_ep = R_ACC;
		aul = r_acc_ep;
		r_acc = divu_w(aul, mloop_fre_Hz);
	}
	if(R_DEC != r_dec_ep)
	{
		r_dec_ep = R_DEC;
		aul = r_dec_ep;
		r_dec = divu_w(aul, mloop_fre_Hz);
	}
	if(I_START != i_start_ep)
	{
		i_start_ep = I_START;
		aul = mulu_w(i_start_ep, KI);
		i_start = divu_w(aul, AMP_RES);
		i_startl = mulu_w(i_start, G_IDRL);
		aul = MC_Cntms(STUP_CUR_RT_MS);
		if(aul > 65535)
		{
			aul = 65535;
		}
		is_rup = divu_w(i_startl, aul);
		aul = MC_Cntms(STUP_CUR_FT_MS);
		if(aul > 65535)
		{
			aul = 65535;
		}
		is_rdw = divu_w(i_startl, aul);
	}
	if(I_MAX != i_max_ep)
	{
		i_max_ep = I_MAX;
		aul = mulu_w(i_max_ep, KI);
		i_max = divu_w(aul, AMP_RES);
#ifdef FLUXWEAK
		id_maxl = mulu_w(i_max, GFW_IDRL);
		aul = MC_Cntms(FW_CUR_RT_MS);
		if(aul > 65535)
		{
			aul = 65535;
		}
		i_del = divu_w(id_maxl, aul);
#endif	// FLUXWEAK
#ifdef MOT_IDENT
		aul = mulu_w(i_max, (HIG_CUR_RATIO * NORMVALF));
		aus = aul >> SH_NORMVAL;
		ial_l1 = mulu_w(aus, G_IDRL);
		aul = mulu_w(i_max, (MID_CUR_RATIO * NORMVALF));
		aus = aul >> SH_NORMVAL;
		ial_l2 = mulu_w(aus, G_IDRL);
		aul = mulu_w(i_max, (LOW_CUR_RATIO * NORMVALF));
		aus = aul >> SH_NORMVAL;
		ial_l3 = mulu_w(aus, G_IDRL);
		aul = MC_Cntms(CUR_RISE_TIME_MS);
		if(aul > 65535)
		{
			aul = 65535;
		}
		ial_r = divu_w(ial_l1, aul);
		ial_l = 0;
#endif	// MOT_IDENT

	}
	if(R_STA != r_sta_ep)
	{
		r_sta_ep = R_STA;
		aul = mulu_w(r_sta_ep, kr_low);
		aul <<= sh_kr_low;
		r_sta = divu_w(aul, OHM_RES);
#ifdef EXACT
		McrpLib_FESetPar(r_sta, l_syn, pm_flx, fb_gain);
#else	// EXACT
		McrpLib_FEASetPar(r_sta, l_syn, flx_ftau);
#endif	// EXACT
	}
	if(L_SYN != l_syn_ep)
	{
		l_syn_ep = L_SYN;
		aul = mulu_w(l_syn_ep, kl_low);
		aul <<= sh_kl_low;
		l_syn = divu_w(aul, HEN_RES);
#ifdef EXACT
		McrpLib_FESetPar(r_sta, l_syn, pm_flx, fb_gain);
#else	// EXACT
		McrpLib_FEASetPar(r_sta, l_syn, flx_ftau);
#endif	// EXACT
	}
	if(PM_FLX != pm_flx_ep)
	{
		pm_flx_ep = PM_FLX;
		aul = mulu_w(pm_flx_ep, kf_low);
		aul <<= sh_kf_low;
		pm_flx = divu_w(aul, WEB_RES);
#ifdef EXACT
		McrpLib_FESetPar(r_sta, l_syn, pm_flx, fb_gain);
#endif	// EXACT
	}
	if(FB_GAIN != fb_gain)
	{
		fb_gain = (int16_t)FB_GAIN;
#ifdef EXACT
		McrpLib_FESetPar(r_sta, l_syn, pm_flx, fb_gain);
#endif	// EXACT
	}

	if(FLX_FTAU != flx_ftau_ep)
	{
		flx_ftau_ep = FLX_FTAU;
		aul = mulu_w(flx_ftau_ep, sam_fre_Hz);
		flx_ftau = divu_w(aul, SEC_RES);
#ifndef EXACT
		McrpLib_FEASetPar(r_sta, l_syn, flx_ftau);
#endif	// EXACT
	}

	kp_cur = (int16_t)KP_CUR;
	aul = mulu_w(KI_CUR, AMP_KISF);
	if((aul >> 15) >= sam_fre_Hz)
	{
		ki_cur = 32767;
	}
	else
	{
		ki_cur = divu_w(aul, sam_fre_Hz);
	}
	kp_vel = (int16_t)KP_VEL;
	aul = mulu_w(KI_VEL, AMP_KISF);
	if((aul >> 15) >= sam_fre_Hz)
	{
		ki_vel = 32767;
	}
	else
	{
		ki_vel = divu_w(aul, sam_fre_Hz);
	}

	if(PHA_OFF != pha_off_ep)
	{
		pha_off_ep = PHA_OFF;
		pha_ofa = mulu_w(pha_off_ep, ((int16_t)(TWOPI / 360.0)));
	}
	if(0 < om_rif2)
	{
		pha_off = pha_ofa;
	}
	else if(0 > om_rif2)
	{
		pha_off = -pha_ofa;
	}
	else
	{
		pha_off = 0;
	}

}

/******************************************************************************
* Function Name: MC_ConFil
* Description  : management of measured values, to give them to the GUI
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_ConFil(void)
{
	int32_t asl;

	// speed reference
	UIF_R.var.rif = rpmrif_y;

	// mechanical speed
	asl = mul_w(om_emf2, korpmdcp);
	mec_rpm = asl >> SH_NORMVAL;
	UIF_R.var.rpm = mec_rpm;

	// imposed frequency
	asl = mul_w(om_emf2, (KOF * 10));
	freq = asl >> SH_NORMVAL;			// Hz amp.10
	UIF_R.var.fre = freq;

	// direct current
	asl = mul_w(idmf, 1000);			// visualization in A/1000
	UIF_R.var.id = div_w(asl, KI);

	// quadrature current
	asl = mul_w(iqmf, 1000);			// visualization in A/1000
	UIF_R.var.iq = div_w(asl, KI);

	// direct voltage
	asl = mul_w(vdcf, 10);				// visualization in V/10
	UIF_R.var.vd = div_w(asl, KV);

	// quadrature voltage
	asl = mul_w(vqcf, 10);				// visualization in V/10
	UIF_R.var.vq = div_w(asl, KV);

	// bus voltage
	asl = mul_w(vbusf, 10);				// visualization in V/10
	UIF_R.var.vb = div_w(asl, KV);

	// total current
	McrpLib_xy_rt(UIF_R.var.id, UIF_R.var.iq, &f_amp_i, &f_arg_i);
	UIF_R.var.toti = f_amp_i;

	// total voltage
	McrpLib_xy_rt(UIF_R.var.vd, UIF_R.var.vq, &f_amp_v, &f_arg_v);
	UIF_R.var.totv = f_amp_v;

	// permanent magnets flux
	asl = mul_w(FluxAmp, WEB_RES);
	asl >>= sh_kf_low;
	UIF_R.var.du0 = div_w(asl, kf_low);

	// alarm flag
	if(0 != UIF_R.var.all)
	{
		SET_ALRM_ON
	}
	else
	{
		RES_ALRM_ON
	}

}

/******************************************************************************
* Function Name: MC_PI
* Description  : PI control with anti-windup
* Arguments    : error err
*                max output mxp, min output mxn
*                proportional gain k_p, integral gain k_i
*                integral term memory address im
* Return Value : output request
******************************************************************************/
static int16_t MC_PI(	int32_t err, int16_t mxp, int16_t mxn,
						int16_t k_p, int16_t k_i, int32_t *im)
{
	int32_t	asl0, asl1;
	int16_t	ass;

	// integral clamp (necessary only if maxp and maxn can be changed)
	asl0 = mul_w(mxp, G_PRO);	// asl0 = mxp; asl0 <<= SH_PRO;
	asl1 = mul_w(mxn, G_PRO);	// asl1 = mxn; asl1 <<= SH_PRO;
	if((*im) > asl0)
	{
		(*im) = asl0;
	}
	else if((*im) < asl1)
	{
		(*im) = asl1;
	}

	if(err > 0)
	{
		// error clamp
		if(err >= 32767)
		{
			ass = 32767;
		}
		else
		{
			ass = err;
		}

		// integral term
		asl1 = mul_w(ass, k_i);
   		asl1 >>= SH_INT;
    	asl1 += (*im);

		// proportional term and total control
		asl0 = mul_w(ass, k_p);
		asl0 += asl1;
		asl0 >>= SH_PRO;

		// final clamp, integral update
		if(asl0 > mxp)
		{
			return(mxp);		// integral not saved due to saturation
		}
		(*im) = asl1;
		//if(asl0 < mxn)		// case not possible, since integral already clamped, then increased
		//{
		//	return(mxn);
		//}
		return( (int16_t)asl0 );
	}
	else if(err < 0)
	{
		// error clamp
		if(err <= -32767)
		{
			ass = 32767;
		}
		else
		{
			ass = -err;
		}

		// integral term
		asl1 = mul_w(ass, k_i);
		asl1 >>= SH_INT;
		asl1 -= *im;
		
		// proportional term and total control
		asl0 = mul_w(ass, k_p);
		asl0 += asl1;
		asl0 >>= SH_PRO;

		// final clamp, integral update
		asl0 = -asl0;
		if(asl0 < mxn)
		{
			return(mxn);		// integral not saved due to saturation
		}
		(*im) = -asl1;
		//if(asl0 > mxp)		// case not possible, since integral already clamped, then decreased
		//{
		//	return(mxp);
		//}
		return( (int16_t)asl0 );
	}
	else
	{
		asl0 = (*im) >> SH_PRO;
		return( (int16_t)asl0 );
	}

}	// end of function MC_PI(...)

#ifdef CPI_TUN
void MC_CurPITun(void)
/***************************************************************************
*	Description:	creation of a current step, to manually tune the
*					current PIs
***************************************************************************/
{
	int32_t asl;
	int16_t ass;

	if(pit_c < 125)
	{
		asl = mul_w(i_max, ratio);
		idr = asl >> SH_NORMVAL;
		iqr = 0;
		if(pit_c < 50)				// 50 samples	0, .., 49, and 50 interrupts
		{			
			asl = mul_w(idm, 1000);
			rbuf[pit_c++] = div_w(asl, KI);
			cur_ampf = 0;
			pit_s = 0;
		}
		else if(pit_c < 65)			// 15 samples	50, .., 64, and 45 interrupts
		{
			if(pit_s < 3)
			{
				++pit_s;
				cur_ampf += idm;
			}
			else
			{
				ass = div_w(cur_ampf, pit_s);
				asl = mul_w(ass, 1000);
				rbuf[pit_c++] = div_w(asl, KI);
				cur_ampf = 0;
				pit_s = 0;
			}
		}
		else if(pit_c < 80)			// 15 samples	65, .., 79, and 135 interrupts
		{
			if(pit_s < 9)
			{
				++pit_s;
				cur_ampf += idm;
			}
			else
			{
				ass = div_w(cur_ampf, pit_s);
				asl = mul_w(ass, 1000);
				rbuf[pit_c++] = div_w(asl, KI);
				cur_ampf = 0;
				pit_s = 0;
			}
		}
		else if(pit_c < 95)			// 15 samples	80, .., 94, and 405 interrupts
		{
			if(pit_s < 27)
			{
				++pit_s;
				cur_ampf += idm;
			}
			else
			{
				ass = div_w(cur_ampf, pit_s);
				asl = mul_w(ass, 1000);
				rbuf[pit_c++] = div_w(asl, KI);
				cur_ampf = 0;
				pit_s = 0;
			}
		}
		else if(pit_c < 110)		// 15 samples	95, .., 109, and 1215 interrupts
		{
			if(pit_s < 81)
			{
				++pit_s;
				cur_ampf += idm;
			}
			else
			{
				ass = div_w(cur_ampf, pit_s);
				asl = mul_w(ass, 1000);
				rbuf[pit_c++] = div_w(asl, KI);
				cur_ampf = 0;
				pit_s = 0;
			}
		}
		else /*if(pit_c < 125)*/	// 15 samples	110, .., 124, and 3645 interrupts
		{
			if(pit_s < 243)
			{
				++pit_s;
				cur_ampf += idm;
			}
			else
			{
				ass = div_w(cur_ampf, pit_s);
				asl = mul_w(ass, 1000);
				rbuf[pit_c++] = div_w(asl, KI);
				cur_ampf = 0;
				pit_s = 0;
			}
		}
	}		// total: 125 samples and 5495 interrupts (around 343ms @16kHz)
	else	// pit_c >= 125
	{
		idr = 0;
		iqr = 0;
		if(pit_c < 150)
		{
			++pit_c;
		}
	}
}
#endif	// CPI_TUN

#ifdef CPI_AUT
void MC_CurPIAuto(void)
/***************************************************************************
*	Description:	creation of a voltage profile, used to automatically
*					tune the current PIs
***************************************************************************/
{
	idr = 0;
	iqr = 0;
	PIT_SetCur(idm, iqm);
	switch(pitun_status)
	{
		case R_UP:		// voltage ramp up till current reaches rated value
			if(PIT_RampUp(&v_req))
			{
				pitun_status = R_MS;
			}
			break;
		case R_MS:		// wait some time, measuring the current level (filtered)
			if(PIT_Measure(&v_req))
			{
				pitun_status = R_DW;
			}
			break;
		case R_DW:		// voltage ramp down
			if(PIT_RampDown(&v_req))
			{
				pitun_status = R_ST;
			}
			break;
		case R_ST:		// voltage step
			if(PIT_VoltStep(&v_req))
			{
				pitun_status = R_EN;
			}
			break;		// finished
		case R_EN:
		default:
			v_req = 0;
			break;
	}
	vdc = v_req;
	vqc = 0;
}
#endif	// CPI_AUT

#ifdef MOT_IDENT
void MC_MotIdent(void)
/***************************************************************************
*	Description:	motor identification sequence
***************************************************************************/
{
	iqr = 0;
	MC_Ident_SetMea(idm, iqm, vdc, vqc, om_eme2);

	if(0 > x_cnt)
	{
		x_cnt = 0;
		ial_l = 0;
		om_iden = 0;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
	}
	else if(X_TIM1 >= x_cnt)	// id ramp up
	{
		if(ial_l < (ial_l1 - ial_r))
		{
			ial_l += ial_r;
		}
		else
		{
			ial_l = ial_l1;
		}
		idr = ial_l >> SH_IDRL;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
	}
	else if(X_TIM2 > x_cnt)		// first measurement cycle
	{
		idr = ial_l1 >> SH_IDRL;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
		MC_Ident_UpdAcc();
	}
	else if(X_TIM2 == x_cnt)	// first measurement acknowledge
	{
		idr = ial_l1 >> SH_IDRL;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
		MC_Ident_Step1();
	}
	else if(X_TIM3 >= x_cnt)	// id ramp dw
	{
		if(ial_l > (ial_l2 + ial_r))
		{
			ial_l -= ial_r;
		}
		else
		{
			ial_l = ial_l2;
		}
		idr = ial_l >> SH_IDRL;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
	}
	else if(X_TIM4 > x_cnt)		// second measurement cycle
	{
		idr = ial_l2 >> SH_IDRL;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
		MC_Ident_UpdAcc();
	}
	else if(X_TIM4 == x_cnt)	// second measurement acknowledge
	{
		idr = ial_l2 >> SH_IDRL;
		om_iden = 0;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
		MC_Ident_Step2();
	}
	else if(X_TIM5 >= x_cnt)	// speed ramp
	{
		idr = ial_l2 >> SH_IDRL;
		if(om_iden < (OM_ID - OM_IDR))
		{
			om_iden += OM_IDR;
		}
		else
		{
			om_iden = OM_ID;
		}
		iden_phasel += om_iden;
		SystemPhase = iden_phasel >> SH_NORMVAL;
		om_eme = om_iden;
		om_eme2 = div_w(om_eme, ko1_2);
	}
	else if(X_TIM6 > x_cnt)		// third measurement cycle
	{
		idr = ial_l2 >> SH_IDRL;
		iden_phasel += om_iden;
		SystemPhase = iden_phasel >> SH_NORMVAL;
		//om_eme = om_iden;
		//om_eme2 = div_w(om_eme, ko1_2);
		MC_Ident_UpdAcc();
	}
	else if(X_TIM6 == x_cnt)	// third measurement acknowledge
	{
		idr = ial_l2 >> SH_IDRL;
		iden_phasel += om_iden;
		SystemPhase = iden_phasel >> SH_NORMVAL;
		//om_eme = om_iden;
		//om_eme2 = div_w(om_eme, ko1_2);
		MC_Ident_Step3();
	}
	else if(X_TIM7 >= x_cnt)	// id ramp dw
	{
		if(ial_l > (ial_l3 + ial_r))
		{
			ial_l -= ial_r;
		}
		else
		{
			ial_l = ial_l3;
		}
		idr = ial_l >> SH_IDRL;
		iden_phasel += om_iden;
		SystemPhase = iden_phasel >> SH_NORMVAL;
		//om_eme = om_iden;
		//om_eme2 = div_w(om_eme, ko1_2);
	}
	else if(X_TIM8 > x_cnt)		// fourth measurement cycle
	{
		idr = ial_l3 >> SH_IDRL;
		iden_phasel += om_iden;
		SystemPhase = iden_phasel >> SH_NORMVAL;
		//om_eme = om_iden;
		//om_eme2 = div_w(om_eme, ko1_2);
		MC_Ident_UpdAcc();
	}
	else if(X_TIM8 == x_cnt)	// fourth measurement acknowledge
	{
		idr = ial_l3 >> SH_IDRL;
		iden_phasel += om_iden;
		SystemPhase = iden_phasel >> SH_NORMVAL;
		//om_eme = om_iden;
		//om_eme2 = div_w(om_eme, ko1_2);
		MC_Ident_Step4();
	}
	else						// waiting end
	{
		ial_l =0;
		om_iden = 0;
		idr = 0;
		om_eme = 0;
		om_eme2 = 0;
		SystemPhase = 0;
	}
	if(0x7FFFFFFF > x_cnt)
	{
		++x_cnt;
	}
}
#endif	// MOT_IDENT

/******************************************************************************
* Function Name: MC_WorkMan
* Description  : Working modes management function (speed ramps etc.)
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_WorkMan(void)
{
	int32_t asl;
	int16_t ass;

	// alarm management
	if(0 != UIF_R.var.all)
	{
		MC_StopOut();
		if(TRIP_ALL == UIF_R.var.all)
		{
			if(	(0 == UIF_W.var.rif) ||
				((0 > rif_mem) && (0 < UIF_W.var.rif)) ||
				((0 < rif_mem) && (0 > UIF_W.var.rif)) ) {
				UIF_R.var.all = 0;
			}
		}
		RES_STARTED
		SET_STOPPED
		rpmrif_x = 0;
		rpmrif_y = 0;
		om_rif2 = 0;
		ramp_status = STOP_0;
		rif_mem = UIF_W.var.rif;
		SET_NORM_MODE		// on alarm, normal mode is reestablished
		RES_STA_BSY
		SET_END_NOK
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
	rif_mem = UIF_W.var.rif;

	// working mode requests management
	if(NOT_COM_REQ)	// all requests are taken into consideration only if COM_OFF
	{
#ifdef CPI_AUT
		if(CURPI_AUT_REQ)
		{
			SET_CURPI_AUT
		}
		else
#endif	// CPI_AUT
#ifdef CPI_TUN
		if(CURPI_TUN_REQ)
		{
			SET_CURPI_TUN
		}
		else
#endif	// CPI_TUN
#ifdef MOT_IDENT
		if(AUTO_IDEN_REQ)
		{
			SET_AUTO_IDEN
		}
#endif	// MOT_IDENT
		if(NOT_NORM_MODE)
		{
			RES_STARTED
			SET_STOPPED
			rpmrif_x = 0;
			rpmrif_y = 0;
			om_rif2 = 0;
			ramp_status = STOP_0;
			if(NORM_MODE_REQ)
			{
				UIF_R.var.kpi = ((uint16_t)KP_CUR);
				UIF_R.var.kii = ((uint16_t)KI_CUR);
				UIF_R.var.rs = ((uint16_t)R_STA);
				UIF_R.var.ls = ((uint16_t)L_SYN);
				UIF_R.var.fl = ((uint16_t)PM_FLX);
				RES_STA_BSY
				SET_NORM_MODE
			}
		}
	}

#ifdef CPI_AUT
	// Automatic current PI tuning mode
	if(CURPI_AUT)
	{
		if(NOT_COM_REQ)
		{
			if(GUI_TRI)
			{
				SET_STA_BSY
				asl = mul_w(i_max, I_PIT_RATIO2);
				ass = asl >> SH_NORMVAL;
				PIT_IniVar(ass, vbusf, SH_PRO, SH_INT);
				v_req = 0;
				pitun_status = R_UP;
				MC_StartOut();
			}
		}
		else if(GUI_STP)
		{
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if(pitun_status >= R_EN)
		{
			MC_StopOut();
			UIF_R.var.kpi = PIT_ReadKP();
			asl = mul_w(PIT_ReadKI(), AMP_KISF);
			if((asl >> 15) >= sam_fre_Hz)
			{
				UIF_R.var.kii = 32767;
			}
			else
			{
				UIF_R.var.kii = div_w(asl, sam_fre_Hz);
			}
			RES_END_NOK
			RES_STA_BSY
		}
		else
		{
			MC_StartOut();
		}
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// CPI_AUT

#ifdef CPI_TUN
	// Manual current PI tuning mode (here normal eeprom parameters are used)
	if(CURPI_TUN)
	{
		if(NOT_COM_REQ)
		{
			if(GUI_TRI)
			{
				SET_STA_BSY
				ratio = mul_w(UIF_W.var.cra, ((int16_t)(0.01 * NORMVAL)));
				if(ratio < 0)
				{
					ratio = 0;
				}
				else if(ratio > NORMVAL)
				{
					ratio = NORMVAL;
				}
				rbuf = ((uint16_t *)(outbuf.ss));
				wbuf = ((uint16_t *)(outbuf1.ss));
				pbuf = rbuf;
				pit_c = 0;
				MC_StartOut();
			}
		}
		else if(GUI_STP)
		{
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if(pit_c >= 130)
		{
			MC_StopOut();
			RES_END_NOK
			RES_STA_BSY
		}
		else
		{
			MC_StartOut();
		}
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// CPI_TUN

#ifdef MOT_IDENT
	// Motor parameters auto identification mode
	if(AUTO_IDEN)
	{
		if(NOT_COM_REQ)
		{
			if(GUI_TRI)
			{
				SET_STA_BSY
				INI_IDENT
				MC_StartOut();
			}
		}
		else if(GUI_STP)
		{
			MC_StopOut();
			SET_END_NOK
			RES_STA_BSY
		}
		else if(IDENT_ENDED)
		{
			MC_StopOut();
			MC_Ident_CalMotPar(&iden_rs, &iden_ls, &iden_fl);
			asl = mul_w(iden_rs, OHM_RES);
			UIF_R.var.rs = div_w(asl, KR);
			asl = mul_w(iden_ls, HEN_RES);
			asl >>= sh_kl_low;
			UIF_R.var.ls = div_w(asl, kl_low);
			asl = mul_w(iden_fl, WEB_RES);
			asl >>= sh_kf_low;
			UIF_R.var.fl = div_w(asl, kf_low);
			RES_END_NOK
			RES_STA_BSY
		}
		else
		{
			MC_StartOut();
		}
		RES_GUI_TRI
		RES_GUI_STP
		return;
	}
#endif	// MOT_IDENT

	// From here normal mode management

	// preliminary reference management
	rif_mem = UIF_W.var.rif;
	ass = UIF_W.var.rif;
	if(ass >= rpm_min)
	{
		if(ass > rpm_max)
		{
			ass = rpm_max;
		}
	}
	else if(ass <= -rpm_min)
	{
		if(ass < -rpm_max)
		{
			ass = -rpm_max;
		}
	}
	else
	{
		ass = 0;
#ifdef DEBUG_OPENLOOP
		if(DEB_OPNLP)
		{
			MC_StopOut();
			rpmrif_x = 0;
			rpmrif_y = 0;
			om_rif2 = 0;
			ramp_status = STOP_0;
			RES_STARTED
			SET_STOPPED
			return;
		}
#endif	// DEBUG_OPENLOOP
	}

#ifdef DCCUR_SETTING
	// DC current vector setting
	if(DCCUR_SET)
	{
		rpmrif_x = 0;
		rpmrif_y = 0;
		om_rif2 = 0;
		ramp_status = STOP_0;
		RES_STARTED
		SET_STOPPED
		if(0 == ass)
		{
			MC_StopOut();
		}
		else
		{
			MC_StartOut();
		}
		return;
	}
#endif	// DCCUR_SETTING

	// ramp management state-machine
	switch(ramp_status)
	{
		case START_P:
			RES_STOPPED
			MC_StartOut();
			rpmrif_x = rpm_min;
			rpmrif_y = rpm_min;
			if(ass < rpm_min)
			{
				ramp_status = STOP_W;
				if(ass <= -rpm_min)
				{
					rpmrif_x = -rpm_min;
				}
				else
				{
					rpmrif_x = 0;
				}
				break;
			}
			else if(NOT_STARTED)
			{
				break;
			}
			ramp_status = GO_P;
			break;
		case GO_P:
			RES_STOPPED
			MC_StartOut();
			if(ass < rpm_min)
			{
				if(rpmrif_y <= rpm_min)
				{
					rpmrif_y = 0;
					ramp_status = STOP_W;
					if(ass <= -rpm_min)
					{
						rpmrif_x = -rpm_min;
					}
					else
					{
						rpmrif_x = 0;
					}
					break;
				}
				rpmrif_x = rpm_min;
			}
			else
			{
				rpmrif_x = ass;
			}
			if(rpmrif_y < (rpmrif_x - r_acc))
			{
				RES_DECEL
				rpmrif_y += r_acc;
			}
			else if(rpmrif_y > (rpmrif_x + r_dec))
			{
				SET_DECEL
				if(NOT_RAMPLIM)
				{
					rpmrif_y -= r_dec;
				}
			}
			else
			{
				RES_DECEL
				rpmrif_y = rpmrif_x;
			}
			break;
		case START_N:
			RES_STOPPED
			MC_StartOut();
			rpmrif_x = -rpm_min;
			rpmrif_y = -rpm_min;
			if(ass > -rpm_min)
			{
				ramp_status = STOP_W;
				if(ass >= rpm_min)
				{
					rpmrif_x = rpm_min;
				}
				else
				{
					rpmrif_x = 0;
				}
				break;
			}
			else if(NOT_STARTED)
			{
				break;
			}
			ramp_status = GO_N;
			break;
		case GO_N:
			RES_STOPPED
			MC_StartOut();
			if(ass > -rpm_min)
			{
				if(rpmrif_y >= -rpm_min)
				{
					rpmrif_y = 0;
					ramp_status = STOP_W;
					if(ass >= rpm_min)
					{
						rpmrif_x = rpm_min;
					}
					else
					{
						rpmrif_x = 0;
					}
					break;
				}
				rpmrif_x = -rpm_min;
			}
			else
			{
				rpmrif_x = ass;
			}
			if(rpmrif_y > (rpmrif_x + r_acc))
			{
				RES_DECEL
				rpmrif_y -= r_acc;
			}
			else if(rpmrif_y < (rpmrif_x - r_dec))
			{
				SET_DECEL
				if(NOT_RAMPLIM)
				{
					rpmrif_y += r_dec;
				}
			}
			else
			{
				RES_DECEL
				rpmrif_y = rpmrif_x;
			}
			break;
		case STOP_W:
			RES_STARTED
			rpmrif_y = 0;
			MC_StartOut();
			if(ass >= rpm_min)
			{
				rpmrif_x = rpm_min;
				if(STOPPED)
				{
					ramp_status = START_P;
				}
			}
			else if(ass <= -rpm_min)
			{
				rpmrif_x = -rpm_min;
				if(STOPPED)
				{
					ramp_status = START_N;
				}
			}
			else
			{
				rpmrif_x = 0;
				if(STOPPED)
				{
					ramp_status = STOP_0;
				}
			}
			break;
		case STOP_0:
		default:
			MC_StopOut();
			SET_STOPPED
			RES_STARTED
			rpmrif_x = 0;
			rpmrif_y = 0;
			if(ass >= rpm_min)
			{
				ramp_status = START_P;
			}
			else if(ass <= -rpm_min)
			{
				ramp_status = START_N;
			}
			else
			{
				ramp_status = STOP_0;
			}
			break;
	}

	// ramp output update and conversion in internal units
	asl = mul_w(rpmrif_y, c_poli);
	asl *= KRPMO;					// long multiply
	om_rif2 = asl >> SH_NORMVAL;

}

int32_t MC_PhaseErr(int32_t speed)
/*******************************************************************************
*	Description:	calc of the phase error due to algorithm medium delay
*					(medium delay is 1.5 sampling periods)
*******************************************************************************/
{
	int32_t    asl;

	asl = speed + (speed >> 1);			// 1.5
    if((asl >= NORMVAL) || (asl <= -NORMVAL))
    {
    	return(asl >> SH_NORMVAL);
	}
	else
	{
        return(0);
	}
}


#ifdef FLUXWEAK
void MC_FluxWeak(int16_t vmf, int32_t irfl, int32_t *il)
/***************************************************************************
*	Description:	d current reference calculation for flux weakening
***************************************************************************/
{

	if(vmf < 0)
	{
		if((*il) > -id_maxl)
			(*il) -= i_del;
	}
	else if((*il) >= (irfl + i_del))
	{
		(*il) -= i_del;
	}
	else if((*il) <= (irfl - i_del))
	{
		(*il) += i_del;
	}
	else
	{
		(*il) = irfl;
	}

}
#endif	// FLUXWEAK

/******************************************************************************
* Function Name: MC_ConInt
* Description  : control interrupt
* Arguments    : none
* Return Value : none
* Note         : this function is called every sampling instant, and performs
*                the motor control
******************************************************************************/
void MC_ConInt(void)
{
	uint32_t	aul;
	int32_t		asl;
	uint16_t	aus;
	int16_t		ass;

	IR(S12AD, S12ADI0) = 0;			// reset interrupt request

	// timing debug
	SET_DL2

	// enables lower priority interrupts (serial communication)
	set_ipl(SCI_INT_P - 1);
	setpsw_i();						// enables other interrupts

	// vbus A/D conversion start
	S12AD.ADCSR.WORD = 0x0400; 		// disable automatic trigger and interrupt
	SEL_VB_AD
	S12AD_START

	// currents calculation while waiting for vbus A/D conversion
	MC_ADMan();			// put measured values in ium, ivm, iwm

	// AUX A/D conversion start
	WAIT_S12AD
	SEL_AUX_AD
	S12AD_START

	// bus voltage calculation
	asl = mul_w(AN6_VB, KADV);
	vbus = asl >> SH_KADV;

	if(COM_ON)
	{

		// current transformations (previous angle)
		McrpLib_uv_alphabeta(ium, ivm, &iam, &ibm);
		McrpLib_alphabeta_dq(iam, ibm, &idm, &iqm);

		// angle management
		McrpLib_AngleSet(SystemPhase);	// from here new angle (sin and cos) (conversions etc.)

		// voltages references and duty cycles calc
#ifdef CPI_AUT
		if(CURPI_AUT)
		{
			MC_CurPIAuto();	// produces directly voltages references vdc, vqc
		}		// if(CURPI_AUT)
		else
#endif	// CPI_AUT
		{	// normal case: idr, iqr come from previous cycle

			/* HERE you can insert decoupling, feed forward, etc. */

			// id control
			asl = mul_w(vfmax, K_MARGV0);
			ass = asl >> SH_NORMVAL;	// vdmax
			asl = idr;
			asl -= idm;
			vdc = MC_PI(asl, ass, -ass, kp_cur, ki_cur, &idint);

			// iq control
			asl = mul_w(vdc, NORMSIN);	// asl = vdc; asl <<= SH_NORMSIN;
			ass = div_w(asl, vfmax);
			ass = McrpLib_sinarcos(ass);
			asl = mul_w(ass, vfmax);
			ass = asl >> SH_NORMSIN;	// vqmax
			asl = iqr;
			asl -= iqm;
			vqc = MC_PI(asl, ass, -ass, kp_cur, ki_cur, &iqint);

		}	// end of normal case

		// voltage transformations (actual angle)
		McrpLib_dq_alphabeta(vdc, vqc, &vac, &vbc);
		McrpLib_alphabeta_uv(vac, vbc, &vuc, &vvc);
		vwc = -vuc - vvc;

#ifdef REFCOMP
		McrpLib_dq_alphabeta(idr, iqr, &iar, &ibr);
		McrpLib_alphabeta_uv(iar, ibr, &iur, &ivr);
		iwr = -iur - ivr;
#endif	// REFCOMP

		// pwm output generation
		MC_PWMGen(); // uses vuc, vvc, vwc and vbus to calculate duty_u, duty_v, duty_w; updates vuc, vvc, vwc values

		// pwm output
		SET_PWM(duty_u, duty_v, duty_w)

		// output voltages in (alpha, beta) frame update after modulation clamping
		McrpLib_uvw_alphabeta(vuc, vvc, vwc, &vac, &vbc);
		McrpLib_alphabeta_dq(vac, vbc, &vdc, &vqc);		// for visualization!!

	}	// else of if(COM_ON)
	else
	{
		iam = 0;
		ibm = 0;
		idm = 0;
		iqm = 0;
		idr = 0;
		iqr = 0;
		idrl = 0;
		iqmax = 0;
#ifdef REFCOMP
		iar = 0;
		ibr = 0;
		iur = 0;
		ivr = 0;
		iwr = 0;
#endif	// REFCOMP
		idint = 0;
		iqint = 0;
		vdc = 0;
		vqc = 0;
		vuc = 0;
		vvc = 0;
		vwc = 0;
		vac = 0;
		vbc = 0;
		duty_u = halfperdead;	// if outputs enabled, this means the lower arm always on (used for BOOTSTRAP)
		duty_v = halfperdead;	// if outputs enabled, this means the lower arm always on (used for BOOTSTRAP)
		duty_w = halfperdead;	// if outputs enabled, this means the lower arm always on (used for BOOTSTRAP)
		cr_status = CR_NO;

		// pwm output
		SET_PWM(duty_u, duty_v, duty_w)

	}

	// timing debug
	//CLR_DL2			// here around 29us after interrupt beginning

	// AUX A/D conversions reading, TMP A/D conversion start
	WAIT_S12AD
	aux_ad = AN10_AUX;
	SEL_TMP_AD
	S12AD_START

	// vbus filtering (faster decrease, slower increase)
	asl = mul_w(vbus, kvout);
	asl >>= SH_NORMVAL;
	asl -= xvbf;
	if(vbus < xvbf)
	{
		 asl <<= SH_VB_UP;			// faster decrease
	}
	xvbm += asl;
	xvbf = xvbm >> SH_VB_INC_LPTK;

	// vfmax calc
	if(xvbf > (MAX_VERR << 1))
		xvbf -= (MAX_VERR << 1);
	else
		xvbf = 0;
#ifdef OVERMOD
	asl = mul_w(xvbf, TWOTHIRDS);	// max out voltage (with hexagonal saturation) is 2*VB/3
#else	// OVERMOD
	asl = mul_w(xvbf, ONEDIVSQRT3);	// max out voltage (without hexagonal saturation) is VB/SQRT3
#endif	// OVERMOD
	vfmax = asl >> SH_S16_CONST;

	// TMP A/D conversion reading
	WAIT_S12AD
	tmp_ad = AN9_TMP;

	// three shunts A/D conversion setting for next cycle
	SEL_CUR_AD
	S12AD.ADCSR.WORD = 0x1600; 		// enable automatic trigger and interrupt

	// startup, position and speed management
	// the outputs of this block are: next system phase, next d and q current references
	if (COM_ON)
	{

#ifdef CPI_TUN
		if(CURPI_TUN)
		{
			MC_CurPITun();
			SystemPhase = 0;
			om_eme = 0;
			om_eme2 = 0;
		}
		else
#endif	// CPI_TUN

#ifdef MOT_IDENT
		if(AUTO_IDEN)
		{
			MC_MotIdent();
		}
		else
#endif	// MOT_IDENT

#ifdef DCCUR_SETTING
		if(DCCUR_SET)
		{
			SystemPhase = 0;
			McrpLib_rt_xy(i_start, pha_ofa, &idr, &iqr);
			om_eme = 0;
			om_eme2 = 0;
		}
		else
#endif	// DCCUR_SETTING

		{	// normal behaviour block beginning

#ifdef FLUXWEAK
			// voltage margin
			asl = mul_w(vfmax, K_MARGV1);
			asl >>= SH_NORMVAL;
			McrpLib_xy_rt(vdc, vqc, &amp_v, &arg_v);
			vmarg = asl - amp_v;
			if( (vmarg < 0) && ( ((om_rff2 > 0) && (iqr <= 0)) || ((om_rff2 < 0) && (iqr >= 0)) ) )
				vmarg = 0;
#endif	// FLUXWEAK

			// speed reference filter
			om_rfm2 += om_rif2;
			om_rfm2 -= om_rff2;
			om_rff2 = om_rfm2 >> SH_OMRIF_LPTK;

			// position estimation
#ifdef EXACT
			McrpLib_FluxEst(vac, vbc, iam, ibm, &FluxAmp, &FluxPhase);
#else	// EXACT
			McrpLib_FluxEstApp(vac, vbc, iam, ibm, &FluxAmp, &FluxPhase);
#endif	// EXACT
			Speed_est = McrpLib_SpeedEst(FluxPhase);
			Speed_est2 = div_w(Speed_est, ko1_2);
			asl = FluxPhase;
			asl += pha_off;
			asl += MC_PhaseErr(Speed_est);
			Phase_est = asl;

#ifdef TRIP_CONTROL
			if(startup_cnt > 3)
			{
				if(trip_cnt >= MAX_TRIP)
				{
					MC_StopOut();
					UIF_R.var.all = TRIP_ALL;
					trip_cnt = 0;
				}
				else if(om_rif2 >= min_speed2)
				{
					if ((Speed_est < min_speed_trip) || (Speed_est > max_speed_trip))
					{
						trip_cnt += 1;
					}
					else if(trip_cnt > 1)
					{
						trip_cnt -= 2;
					}
				}
				else if(om_rif2 <= (-min_speed2))
				{
					if((Speed_est > (-min_speed_trip)) || (Speed_est < (-max_speed_trip)))
					{
						trip_cnt += 1;
					}
					else if(trip_cnt > 1)
					{
						trip_cnt -= 2;
					}
				}
				else
				{
					if((Speed_est > min_speed_trip) || (Speed_est < (-min_speed_trip)))
					{
						trip_cnt += 1;
					}
					else if(trip_cnt)
					{
						trip_cnt -= 1;
					}
				}
			}
			else
			{
				trip_cnt = 0;
			}	// end trip control
#endif	// TRIP_CONTROL

			// STARTUP and normal control
			if(ramp_status <= STOP_W)
			{
				iqr = 0;
				idr = 0;
				idrl = 0;
				startup_cnt = 0;
				RES_STARTED
				SET_STOPPED
				SystemPhase = 0;
				startup_phase = 0;
				startup_phasel = 0;
				om_stup = 0;
				om_eme = 0;
				om_eme2 = 0;
			}
			else if(0 == startup_cnt)		// STARTUP 0 control: current ramp till startup value at speed=0
			{
				iqr = 0;
				SystemPhase = 0;
				startup_phase = 0;
				startup_phasel = 0;
				om_stup = 0;
				om_eme = 0;
				om_eme2 = 0;
				if(idrl > (i_startl + is_rup))
				{
					idrl -= is_rup;
				}
				else if(idrl < (i_startl - is_rup))
				{
					idrl += is_rup;
				}
				else
				{
					idrl = i_startl;
					startup_cnt = 1;
				}
				idr = idrl >> SH_IDRL;
			}
			else if(1 == startup_cnt)	// STARTUP 1 control: speed ramp till minimum speed at maximum current
			{

				iqr = 0;
				idr = i_start;

				if(START_P == ramp_status)
				{
					if(om_stup < (om_chg - delta_om))
					{
						om_stup += delta_om;
					}
					else
					{
						om_stup = om_chg;
						startup_cnt = 2;
					}
				}
				else if(START_N == ramp_status)
				{
					if(om_stup > (-om_chg + delta_om))
					{
						om_stup -= delta_om;
					}
					else
					{
						om_stup = -om_chg;
						startup_cnt = 2;
					}
				}

				// feed-forward angle management
				startup_phasel += om_stup;
				startup_phase = startup_phasel >> SH_NORMVAL;
				SystemPhase = startup_phase;
				om_eme = om_stup;
				om_eme2 = div_w(om_eme, ko1_2);
			}
#ifdef DEBUG_OPENLOOP
			else if(DEB_OPNLP)
			{
				iqr = 0;
				idr = i_start;
				startup_phasel += om_stup;
				startup_phase = startup_phasel >> SH_NORMVAL;
				SystemPhase = startup_phase;
				om_eme = om_stup;
				om_eme2 = div_w(om_eme, ko1_2);
			}
#endif	// DEBUG_OPENLOOP
			else if(2 == startup_cnt)	// STARTUP 2 control: reference system change
			{
				startup_cnt = 3;

				// control memories calc and setting
				vdc = idint >> SH_PRO;
				vqc = iqint >> SH_PRO;
				McrpLib_dq_alphabeta(vdc, vqc, &vac, &vbc);	// current PI integral terms in old ref. frame
				McrpLib_dq_alphabeta(idr, iqr, &iam, &ibm);
				McrpLib_AngleSet(Phase_est);				// phase update
				McrpLib_alphabeta_dq(vac, vbc, &vdc, &vqc);
				idint = mul_w(vdc, G_PRO);					// d current PI integral term in new ref. frame
				iqint = mul_w(vqc, G_PRO);					// q current PI integral term in new ref. frame
				McrpLib_alphabeta_dq(iam, ibm, &idr, &iqr);
				idrl = mul_w(idr, G_IDRL);					// idrl = idr; idrl <<= SH_IDRL;
				errint = mul_w(iqr, G_PRO);					// speed PI integral term

				// phase (and speed) alignment
				SystemPhase = Phase_est;
				Speed_est = om_stup;
				Speed_est2 = div_w(om_eme, ko1_2);
				McrpLib_SpeedSet(Speed_est, Phase_est);
				om_eme = Speed_est;
				om_eme2 = Speed_est2;
				om_rff2 = om_eme2;
				om_rfm2 = om_rff2;
				om_rfm2 <<= SH_OMRIF_LPTK;
				trip_cnt = 0;
				SET_STARTED
			}
			else if(3 == startup_cnt)	// Startup 3: idr decreasing till rated value (0)
			{

				// phase and speed update
				SystemPhase = Phase_est;
				om_eme = Speed_est;
				om_eme2 = Speed_est2;

				// speed control
				asl = om_rff2;
				asl -= om_eme2;
				iqr = MC_PI(asl, i_max, -i_max, kp_vel, ki_vel, &errint);

				// idmax calc and d reference clamping
				asl = mulu_w(MC_VecComp(i_max, iqm), G_IDRL);	// asl = MC_VecComp(i_max, iqr); asl <<= SH_IDRL;
				if(idrl > asl)
				{
					idrl = asl;
				}
				else if(idrl < -asl)
				{
					idrl = -asl;
				}

				// idr ramp
				if(idrl > is_rdw)
				{
					idrl -= is_rdw;
				}
				else if(idrl < -is_rdw)
				{
					idrl += is_rdw;
				}
				else
				{
					idrl = 0;
					startup_cnt = 4;
				}
				idr = idrl >> SH_IDRL;

			}	// end of startup 3
			else		// normal sensorless control
			{

				// phase and speed update
				SystemPhase = Phase_est;
				om_eme = Speed_est;
				om_eme2 = Speed_est2;

				// idr and iqmax calc
#ifdef FLUXWEAK
				MC_FluxWeak(vmarg, 0, &idrl);
				idr = idrl >> SH_IDRL;
				iqmax = MC_VecComp(i_max, idr);
#else	// FLUXWEAK
				idr = 0;
				iqmax = i_max;	// iqmax = MC_VecComp(i_max, idm);
#endif	// FLUXWEAK

				// speed control
				asl = om_rff2;
				asl -= om_eme2;
				iqr = MC_PI(asl, iqmax, -iqmax, kp_vel, ki_vel, &errint);

			}	// end of normal control

		}	// normal behaviour block end

	}	// end of if(COM_ON)
	else
	{
#ifdef FLUXWEAK
		vmarg = vfmax;
		amp_v = 0;
		arg_v = 0;
#endif	// FLUXWEAK
		om_rfm2 = 0;
		om_rff2 = 0;
		startup_cnt = 0;
		SystemPhase = 0;
		FluxAmp = pm_flx;
		FluxPhase = SystemPhase;
#ifdef EXACT
		McrpLib_FluxSet(0, 0, 0, 0, FluxAmp, FluxPhase);
#else	// EXACT
		McrpLib_FluxSetApp(0, 0, 0, 0, FluxAmp, FluxPhase);
#endif	// EXACT
		Phase_est = FluxPhase;
		Speed_est = 0;
		Speed_est2 = 0;
		McrpLib_SpeedSet(Speed_est, Phase_est);
		startup_phasel = 0;
		startup_phase = 0;
		om_eme = 0;
		om_eme2 = 0;
		om_stup = 0;
		errint = 0;
	}	// end else of if(COM_ON)

	// filters
	if(COM_ON)
	{
		idmm += idm;
		idmm -= idmf;
		idmf = idmm >> SH_VISU_LPTK;
		iqmm += iqm;
		iqmm -= iqmf;
		iqmf = iqmm >> SH_VISU_LPTK;
		vdcm += vdc;
		vdcm -= vdcf;	
		vdcf = vdcm >> SH_VISU_LPTK;
		vqcm += vqc;
		vqcm -= vqcf;
		vqcf = vqcm >> SH_VISU_LPTK;
		om_emm2 += om_eme2;
		om_emm2 -= om_emf2;
		om_emf2 = om_emm2 >> SH_VISU_LPTK;
	}
	else
	{
		idmm = 0;
		iqmm = 0;
		vdcm = 0;
		vqcm = 0;
		om_emm2 = 0;
		idmf = 0;
		iqmf = 0;
		vdcf = 0;
		vqcf = 0;
		om_emf2 = 0;
	}
	vbusm += vbus;
	vbusm -= vbusf;
	vbusf = vbusm >> SH_VISU_LPTK;

	// analog output management
#ifdef CPI_TUN
	if(CURPI_TUN)
	{
		if(idr > 0)
		{
			asl = mul_w(idm, 16384);
			asl = div_w(asl, idr);
			asl += 8192;
			asl <<= 1;
			MC_Set_PWM_TST((uint16_t)asl);
		}
		else
		{
			MC_Set_PWM_TST(16384);
		}
	}
	else
#endif	// CPI_TUN
	{
		MC_Set_PWM_TST((uint16_t)SystemPhase);
	}

#ifdef OSC_WIN
	// management of oscilloscope window in GUI
	if(NORM_MODE)
	{
		if(CSELD)
		{
			asl = mul_w(ium, 1000);
			sample = div_w(asl, KI);		// mA
		}
		else
		{
			asl = mul_w(SystemPhase, 1571);	// 1571=NORMVAL*(2pi*1000/65536)
			sample = asl >> SH_NORMVAL;
		}
		if(COM_ON)
		{
			if(NOT_TRGED)
			{
				if(selmem != ((uint16_t)UIF_W.var.sel))
				{
					selmem = (uint16_t)UIF_W.var.sel;
					bcntm = selmem >> 8;
					if(0 == (selmem & 0x00FF))
					{
						RES_CSELD
					}
					else
					{
						SET_CSELD
					}
				}
				if(	((om_eme2 > 0) && (phmem < 0) && (SystemPhase >= 0)) ||
					((om_eme2 < 0) && (phmem > 0) && (SystemPhase <= 0)) )
				{
					SET_TRGED
					bcnt1 = 1;
					bcnt0 = 1;
					wbuf[0] = sample;
				}
			}
			else if(bcnt0 < OUTBUF_SDIM)
			{
				if(bcnt1 < bcntm)
				{
					++bcnt1;
				}
				else
				{
					bcnt1 = 1;
					wbuf[bcnt0++] = sample;
				}
			}
			else if(NOT_RBUSY)
			{
				pbuf = rbuf;
				rbuf = wbuf;
				wbuf = pbuf;
				RES_TRGED
			}
		}	// COM_ON
		else
		{
			RES_TRGED
			if(selmem != ((uint16_t)UIF_W.var.sel))
			{
				selmem = (uint16_t)UIF_W.var.sel;
				bcntm = selmem >> 8;
				if(0 == (selmem & 0x00FF))
				{
					RES_CSELD
				}
				else
				{
					SET_CSELD
				}
			}
			if(bcnt0 < OUTBUF_SDIM)
			{
				wbuf[bcnt0++] = 0;
			}
			else if(NOT_RBUSY)
			{
				pbuf = rbuf;
				rbuf = wbuf;
				wbuf = pbuf;
				bcnt0 = 0;
			}
		}	// COM_OFF
		phmem = SystemPhase;
	}	// NORM_MODE
#endif	// OSC_WIN

	// serial communication receive delay management
	UI_RTX232_RXDelMan();

	// main loop synchronization counter
	if(cnt_int)
	{
		--cnt_int;
	}

	// timing debug
	CLR_DL2

	// interrupt end
	return;

}

/******************************************************************************
* Function Name: MC_FaultInt
* Description  : interrupt routine for POE (/fault from power stage)
* Arguments    : none
* Return Value : none
* Note         : this function stops the PWM output
******************************************************************************/
void MC_FaultInt(void)
{
	MTU.TOER.BYTE = 0xC0;			// all outputs disabled (at inactive level)
	UIF_R.var.all = FAULT_ALL;
	SET_ALRM_ON
	MC_StopOut();

	IEN(POE, OEI1) = 0;				// interrupt disable
	IR(POE, OEI1) = 0;				// reset interrupt request

	POE.ICSR1.BIT.PIE1 = 0;			// interrupt request disable
	if(0 != POE.ICSR1.BIT.POE1F)
	{
		POE.ICSR1.BIT.POE1F = 0;	// reset flag
	}

	POE.OCSR1.BIT.OIE1 = 0;			// interrupt request disable
	if(0 != POE.OCSR1.BIT.OSF1)
	{
		POE.OCSR1.BIT.OSF1 = 0;		// reset flag
	}

}

